'
' MediaMonkey Script
'
' NAME: BatchArtFinder 3.0
'
' AUTHOR: trixmoto (http://trixmoto.net)
' DATE : 28/03/2009
'
' INSTALL: Copy to Scripts\Auto directory and add the following to Scripts.ini 
'          Don't forget to remove comments (') and set the order appropriately
'
' Thanks to Zachm019 for his beta testing which helped a lot
' Thanks to Ryedunn for finding and fixing change with Google results
' Thanks to Austin Powers for his UTF encoding
'
' NOTE: This script automatically registers the MSInet component to download the images
'
' [BatchArtFinder]
' FileName=Auto\BatchArtFinder.vbs
' ProcName=BatchArtFinder
' Order=28
' DisplayName=Batch Art Finder
' Description=Find album art in batch
' Language=VBScript
' ScriptType=0 
'
' FIXES: Added extra debug messaging and timestamps
'        Fixed errors when track path is not a file path
'        Removed previous workaround for FLAC and OGG files
'        Fixed option to process albums with artwork
'        Fixed "still executing last request" errors
'        Fixed "starting a transaction while some query is open" errors
'        Fixed "access denied" errors
'        Fixed multiple searches happening concurrently
'        Added option to automatically try next images after saving
'        Added links to select all/none/inverse and try next image for all
'        Fixed using default image in case of error
'        Added check for images downloading as html error pages
'        Fixed using set image display size breaks min/max width/height options
' 

Option Explicit
Dim FileMask,ImageSize,Storage,Extra,Singles,Maximum,Include,Persist
Dim PanelType,Toolbar,Default,StartID,CheckAll,Source,MinW,MinH,MaxW,MaxH
Dim ConfMode,SaveMode,AutoMode,IncludeDone,SetSize,SetW,SetH,FileTypes
Dim Debug : Debug = False

Sub onStartup
  'set default values
  Dim ini : Set ini = SDB.IniFile
  If ini.StringValue("BatchArtFinder","Storage") = "" Then
    ini.StringValue("BatchArtFinder","Storage") = 1 '0=None 1=InTag 2=AsLink 3=Both 4=InTagDeleteFile 5=DownloadOnly
  End If
  If ini.StringValue("BatchArtFinder","ImageSize") = "" Then
    ini.StringValue("BatchArtFinder","ImageSize") = 1 'size of image to find
  End If
  If ini.StringValue("BatchArtFinder","FileMask") = "" Then
    ini.StringValue("BatchArtFinder","FileMask") = "<path>\<album>" 'mask for saving images
  End If
  If ini.StringValue("BatchArtFinder","Extra") = "" Then
    ini.StringValue("BatchArtFinder","Extra") = "" 'extra sql in where statement
  End If
  If ini.StringValue("BatchArtFinder","Singles") = "" Then
    ini.StringValue("BatchArtFinder","Singles") = "0" 'include non-album tracks
  End If
  If ini.StringValue("BatchArtFinder","Maximum") = "" Then
    ini.StringValue("BatchArtFinder","Maximum") = "0" 'maximum albums per batch
  End If  
  If ini.StringValue("BatchArtFinder","Include") = "" Then
    ini.StringValue("BatchArtFinder","Include") = "0" 'include single titles in search url
  End If  
  If ini.StringValue("BatchArtFinder","PanelType") = "" Then
    ini.StringValue("BatchArtFinder","PanelType") = "1" '1=StayOnTop 2=Normal 3=Dockable
  End If
  If ini.StringValue("BatchArtFinder","Toolbar") = "" Then
    ini.StringValue("BatchArtFinder","Toolbar") = "1" 'add toolbar button
  End If  
  If ini.StringValue("BatchArtFinder","Default") = "" Then
    ini.StringValue("BatchArtFinder","Default") = "0" 'add none.jpg to images without art
  End If  
  If ini.StringValue("BatchArtFinder","StartID") = "" Then
    ini.StringValue("BatchArtFinder","StartID") = "0"
  End If  
  If ini.StringValue("BatchArtFinder","CheckAll") = "" Then
    ini.StringValue("BatchArtFinder","CheckAll") = "0"
  End If   
  If ini.StringValue("BatchArtFinder","Source") = "" Then
    ini.StringValue("BatchArtFinder","Source") = "0" '0=google 1=rateyourmusic 2=juno 3=albumart 4=lastfm 5=localhost
  End If  
  If ini.StringValue("BatchArtFinder","MinW") = "" Then
    ini.StringValue("BatchArtFinder","MinW") = "0" 'pixels
  End If   
  If ini.StringValue("BatchArtFinder","MinH") = "" Then
    ini.StringValue("BatchArtFinder","MinH") = "0" 'pixels
  End If   
  If ini.StringValue("BatchArtFinder","MaxW") = "" Then
    ini.StringValue("BatchArtFinder","MaxW") = "9999" 'pixels
  End If   
  If ini.StringValue("BatchArtFinder","MaxH") = "" Then
    ini.StringValue("BatchArtFinder","MaxH") = "9999" 'pixels
  End If 
  If ini.StringValue("BatchArtFinder","ConfMode") = "" Then
    ini.StringValue("BatchArtFinder","ConfMode") = "0" '0=prompt 1=yes 2=all
  End If  
  If ini.StringValue("BatchArtFinder","SaveMode") = "" Then
    ini.StringValue("BatchArtFinder","SaveMode") = "0" '0=nothing 1=next 2=close 3=trynext
  End If  
  If ini.StringValue("BatchArtFinder","AutoMode") = "" Then
    ini.StringValue("BatchArtFinder","AutoMode") = "0"
  End If
  If ini.StringValue("BatchArtFinder","SetSize") = "" Then
    ini.StringValue("BatchArtFinder","SetSize") = "0"
  End If
  If ini.StringValue("BatchArtFinder","SetW") = "" Then
    ini.StringValue("BatchArtFinder","SetW") = "100" 'pixels
  End If
  If ini.StringValue("BatchArtFinder","SetH") = "" Then
    ini.StringValue("BatchArtFinder","SetH") = "100" 'pixels
  End If  
  If ini.StringValue("BatchArtFinder","FileTypes") = "" Then
    ini.StringValue("BatchArtFinder","FileTypes") = "jpg"
  End If    
  If ini.StringValue("BatchArtFinder","Persist") = "" Then
    ini.StringValue("BatchArtFinder","Persist") = "0"
  End If      
  
  'set default sizes
  If ini.StringValue("BatchArtFinder","PanelDockedTo") = "" Then
    ini.StringValue("BatchArtFinder","PanelDockedTo") = "4"
  End If
  If ini.StringValue("BatchArtFinder","PanelWidth") = "" Then
    ini.StringValue("BatchArtFinder","PanelWidth") = "600"
  End If
  If ini.StringValue("BatchArtFinder","PanelHeight") = "" Then
    ini.StringValue("BatchArtFinder","PanelHeight") = "400"
  End If   
  If ini.StringValue("BatchArtFinder","PanelTop") = "" Then
    ini.StringValue("BatchArtFinder","PanelTop") = "100"
  End If
  If ini.StringValue("BatchArtFinder","PanelLeft") = "" Then
    ini.StringValue("BatchArtFinder","PanelLeft") = "100"
  End If  
  
  'add option sheet
  Call SDB.UI.AddOptionSheet("BatchArtFinder Settings",Script.ScriptPath,"InitSheet","SaveSheet",-3)
  
  'add toolbar button
  Dim but : Set but = SDB.Objects("BAFToolbarButton")
  If (but Is Nothing) Then
    Set but = SDB.UI.AddMenuItem(SDB.UI.Menu_TbStandard,0,0)
    but.Caption = "Batch Art Finder"
    but.IconIndex = SDB.RegisterIcon("Scripts\Auto\baf.ico",0)
    but.UseScript = Script.ScriptPath
    but.OnClickFunc = "ToolbarArtFinder"
    Set SDB.Objects("BAFToolbarButton") = but
  End If
  If ini.IntValue("BatchArtFinder","Toolbar") = 1 Then
    but.Visible = True
  Else
    but.Visible = False
  End If  
  
  'save options for future loops
  Dim tmp : tmp = ""
  Dim rep : Set rep = SDB.Objects("BatchArtFinderReplies")
  If rep Is Nothing Then
    Set rep = CreateObject("Scripting.Dictionary")
    If ini.IntValue("BatchArtFinder","Persist") = 1 Then
      tmp = ini.StringValue("BatchArtFinder","Replies1")
      If Not (tmp = "") Then
        rep.Item("restrict") = tmp
      End If
      tmp = ini.StringValue("BatchArtFinder","Replies2")
      If Not (tmp = "") Then
        rep.Item("beginning") = tmp
      End If
      tmp = ini.StringValue("BatchArtFinder","Replies3")
      If Not (tmp = "") Then
        rep.Item("include") = tmp
      End If
    End If
    Set SDB.Objects("BatchArtFinderReplies") = rep
  End If  
  If ini.IntValue("BatchArtFinder","Persist") = 1 Then
    tmp = GetHint()
    If Not (tmp = "") Then
      but.Caption = "Batch Art Finder"&Chr(13)&"-"&Chr(13)&tmp
    End If
  End If  
  
  'add event for auto mode
  Call Script.RegisterEvent(SDB,"OnPlay","AutoArtFinder")
End Sub

Sub ToolbarArtFinder(ClickedBtn)
  SDB.IniFile.BoolValue("BatchArtFinder","IncludeDone") = False
  If Not (SDB.IniFile.IntValue("BatchArtFinder","Persist") = 1) Then
    Set SDB.Objects("BatchArtFinderReplies") = Nothing
  End If
  Call DoBatchArtFinder(Nothing)
End Sub

Sub AutoArtFinder()
  If SDB.IniFile.IntValue("BatchArtFinder","AutoMode") = 1 Then
    Dim itm : Set itm = SDB.Player.CurrentSong
    If Not (itm Is Nothing) Then
      If itm.Album.AlbumArt Is Nothing Then 
        SDB.IniFile.BoolValue("BatchArtFinder","IncludeDone") = False
        If Not (SDB.IniFile.IntValue("BatchArtFinder","Persist") = 1) Then
          Set SDB.Objects("BatchArtFinderReplies") = Nothing
        End If
        Call DoBatchArtFinder(itm)
      End If
    End If
  End If
End Sub

Sub BatchArtFinder()
  SDB.IniFile.BoolValue("BatchArtFinder","IncludeDone") = False
  If Not (SDB.IniFile.IntValue("BatchArtFinder","Persist") = 1) Then
    Set SDB.Objects("BatchArtFinderReplies") = Nothing
  End If
  Call DoBatchArtFinder(Nothing)
End Sub

Sub DoBatchArtFinder(itm)
  'check script is not already running
  Dim form : Set form = SDB.Objects("BatchArtFinderForm")
  If Not (form Is Nothing) Then 
    Exit Sub 
  End If

  'setup variables
  Dim ini : Set ini = SDB.IniFile  
  Singles = ini.IntValue("BatchArtFinder","Singles")
  Maximum = ini.IntValue("BatchArtFinder","Maximum")
  Extra = ini.StringValue("BatchArtFinder","Extra")
  PanelType = ini.IntValue("BatchArtFinder","PanelType")
  CheckAll = ini.IntValue("BatchArtFinder","CheckAll")
  Source = ini.IntValue("BatchArtFinder","Source")
  ConfMode = ini.IntValue("BatchArtFinder","ConfMode")
  IncludeDone = ini.BoolValue("BatchArtFinder","IncludeDone")
  Persist = ini.IntValue("BatchArtFinder","Persist")
  Dim chk : chk = ""
  If CheckAll = 1 Then
    chk = " checked"
  End If  
 
  'create form
  If PanelType = 1 Or PanelType = 2 Then
    Set form = SDB.UI.NewForm
    form.Common.SetRect 100, 100, 100, 100
    form.BorderStyle = 2
    form.FormPosition = 4
    If PanelType = 1 Then
      form.StayOnTop = True
    End If
    form.SavePositionName = "BatchArtFinderPosition"
  ElseIf PanelType = 3 Then
    Set form = SDB.UI.NewDockablePanel 
    form.Common.Visible = False
    Dim dock : dock = ini.IntValue("BatchArtFinder","PanelDockedTo")
    If (dock > 0) And (dock < 5) Then
      form.DockedTo = dock
    Else
      form.DockedTo = 4
    End If  
  Else
    Exit Sub
  End If
  form.Common.Width = ini.IntValue("BatchArtFinder","PanelWidth")
  form.Common.Height = ini.IntValue("BatchArtFinder","PanelHeight")  
  form.Common.Top = ini.IntValue("BatchArtFinder","PanelTop")
  form.Common.Left = ini.IntValue("BatchArtFinder","PanelLeft")
  form.Common.ControlName = "BatchArtFinder"
  form.Caption = "Batch Art Finder"
  Script.RegisterEvent form, "OnClose", "FormClose"
  Set SDB.Objects("BatchArtFinderForm") = form
  Set SDB.Objects("BAFCurrentSong") = itm
  
  'save options for future loops
  Dim rep : Set rep = SDB.Objects("BatchArtFinderReplies")
  If rep Is Nothing Then
    Set rep = CreateObject("Scripting.Dictionary")
    If Persist = 1 Then
      tmp = ini.StringValue("BatchArtFinder","Replies1")
      If Not (tmp = "") Then
        rep.Item("restrict") = tmp
      End If
      tmp = ini.StringValue("BatchArtFinder","Replies2")
      If Not (tmp = "") Then
        rep.Item("beginning") = tmp
      End If
      tmp = ini.StringValue("BatchArtFinder","Replies3")
      If Not (tmp = "") Then
        rep.Item("include") = tmp
      End If
    End If
    Set SDB.Objects("BatchArtFinderReplies") = rep
  End If  
  
  'create todo list  
  Dim todo : Set todo = CreateObject("Scripting.Dictionary")
  Dim list : Set list = SDB.SelectedSongList
  Dim list2 : Set list2 = SDB.NewSongList
  Dim res : res = 0
  Dim total : total = 0
  Dim zero : zero = 0
  Dim hnt : hnt = ""
  If itm Is Nothing Then
    'manual
    If Not (list Is Nothing) Then
      If list.Count = 0 Then 
        Set list = Nothing
      Else
        For res = 0 To list.Count-1
          If list.Item(res).Album.ID = 0 Then
            zero = 1
            If Singles = 1 Then
              Call list2.Add(list.Item(res))
            Else
              Exit For
            End If
          End If
        Next     
        total = list.Albums.Count + list2.Count - zero
        Select Case ConfMode
          Case 1
            res = mrYes
          Case 2
            res = mrAll
          Case Else
            If rep.Exists("restrict") Then
              res = Int(rep.Item("restrict"))
            Else
              res = SDB.MessageBox("Batch Art Finder - do you wish to restrict the search to the "&total&" selected albums?",mtConfirmation,Array(mbYes,mbAll,mbCancel))
              rep.Item("restrict") = res
              If Persist = 1 Then
                ini.StringValue("BatchArtFinder","Replies1") = res
                hnt = GetHint()
                If Not (hnt = "") Then
                  hnt = Chr(13)&"-"&Chr(13)&hnt
                End If
                SDB.Objects("BAFToolbarButton").Caption = "Batch Art Finder"&hnt
              End If
            End If
        End Select
        Select Case res
          Case mrYes
            'use list
          Case mrAll
            Set list = Nothing
          Case Else
            form.Common.ControlName = ""
            Set SDB.Objects("BatchArtFinderForm") = Nothing
            Exit Sub
        End Select
      End If
    End If  
  Else
    'automatic
    Set list = SDB.NewSongList
    Call list.Add(itm)
    ini.IntValue("BatchArtFinder","StartID") = 0
  End If
  
  'get start position  
  Dim temp : temp = ini.StringValue("BatchArtFinder","StartID")
  If Left(temp,1) = "*" Then
    StartID = Int(Mid(temp,2))
  Else
    If temp = "0" Then
      StartID = 0
    Else
      If rep.Exists("beginning") Then
        res = Int(rep.Item("beginning"))
      Else
        res = SDB.MessageBox("Batch Art Finder - do you wish to start from the beginning? (No will continue from your last run).",mtConfirmation,Array(mbYes,mbNo,mbCancel))
        rep.Item("beginning") = res
        If Persist = 1 Then
          ini.StringValue("BatchArtFinder","Replies2") = res
          hnt = GetHint()
          If Not (hnt = "") Then
            hnt = Chr(13)&"-"&Chr(13)&hnt
          End If
          SDB.Objects("BAFToolbarButton").Caption = "Batch Art Finder"&hnt          
        End If
      End If
      Select Case res
        Case mrYes
          StartID = 0
        Case mrNo
          StartID = Int(temp)
        Case Else
          form.Common.ControlName = ""
          Set SDB.Objects("BatchArtFinderForm") = Nothing
          Exit Sub
      End Select      
    End If
  End If   
  ini.IntValue("BatchArtFinder","StartID") = StartID
  
  'create hidden browser
  Dim wb : Set wb = SDB.UI.NewActiveX(form, "Shell.Explorer") 
  wb.Common.SetRect 10,10,10,10
  Set SDB.Objects("BatchArtFinderShell") = wb.Interf
  
  'create results browser
  Set wb = SDB.UI.NewActiveX(form, "Shell.Explorer") 
  wb.Common.Align = 5  
  
  'create panel (for buttons)
  Dim foot : Set foot = SDB.UI.NewPanel(form)
  foot.Common.Align = 2
  foot.Common.Height = 25 

  'create source dropdown  
  Dim drp1 : Set drp1 = SDB.UI.NewDropDown(foot)
  drp1.Common.SetRect 0,2,100,25
  drp1.Common.Anchors = 4+8
  drp1.Common.ControlName = "drp1"
  drp1.Style = 2
  drp1.AddItem("Google")
  drp1.AddItem("RateYourMusic")
  drp1.AddItem("Juno")
  drp1.AddItem("AlbumArt")
  drp1.AddItem("AudioScrobbler")
  drp1.AddItem("LocalHost")
  '!
  drp1.ItemIndex = Source 
  drp1.UseScript = Script.ScriptPath
  drp1.OnSelectFunc = "SourceClick"      
  Set SDB.Objects("BatchArtFinderDrp1") = drp1
  
  'create save button
  Dim btn1 : Set btn1 = SDB.UI.NewButton(foot)
  btn1.Caption = "Save checked images"
  btn1.Common.SetRect 0,0,140,25 
  btn1.Common.Anchors = 4+8
  btn1.Common.ControlName = "btn1"
  btn1.UseScript = Script.ScriptPath
  btn1.OnClickFunc = "SaveClick"  
  Set SDB.Objects("BatchArtFinderBtn1") = btn1
  
  'create pause button
  Dim btn2 : Set btn2 = SDB.UI.NewButton(foot)
  btn2.Caption = "Pause"
  btn2.Common.SetRect 0,0,65,25
  btn2.Common.Anchors = 4+8
  btn2.Common.ControlName = "btn2"
  btn2.UseScript = Script.ScriptPath
  btn2.OnClickFunc = "PauseClick"
  Set SDB.Objects("BatchArtFinderBtn2") = btn2
  
  'create next button
  If list Is Nothing Then
    Dim btn3 : Set btn3 = SDB.UI.NewButton(foot)
    btn3.Caption = "Next"
    btn3.Common.SetRect 0,0,65,25
    btn3.Common.Anchors = 4+8
    btn3.Common.ControlName = "btn3"
    btn3.UseScript = Script.ScriptPath
    btn3.OnClickFunc = "NextClick"
    Set SDB.Objects("BatchArtFinderBtn3") = btn3
  End If  
    
  'create status label
  Dim lbl1 : Set lbl1 = SDB.UI.NewLabel(foot)
  lbl1.Common.Top = 7
  lbl1.Common.Left = 103
  lbl1.Common.ControlName = "lbl1"
  lbl1.Caption = ""  
  
  'create logfile
  Dim wsh : Set wsh = CreateObject("WScript.Shell")
  Dim paths : Set paths = CreateObject("Scripting.Dictionary")
  paths.Item("log") = wsh.ExpandEnvironmentStrings("%TEMP%")&"\BatchArtFinder.log"
  paths.Item("htm") = wsh.ExpandEnvironmentStrings("%TEMP%")&"\BatchArtFinder.htm"
  Set SDB.Objects("BatchArtFinderPaths") = paths
  Dim fso : Set fso = SDB.Tools.FileSystem
  Dim logf : Set logf = Nothing
  If Debug Then
    Set logf = fso.CreateTextFile(paths.Item("log"),True)
    logf.WriteLine Time&" Batch Art Finder started..."
    logf.WriteLine Time&" Singles="&Singles
    logf.WriteLine Time&" Maximum="&Maximum
    logf.WriteLine Time&" Extra="&Extra
    logf.WriteLine Time&" PanelType="&PanelType
    logf.WriteLine Time&" CheckAll="&CheckAll
    logf.WriteLine Time&" Source="&Source
    logf.WriteLine Time&" ConfMode="&ConfMode
    logf.WriteLine Time&" IncludeDone="&IncludeDone
    logf.WriteLine Time&" Persist="&Persist
  Else 
    'remove old logfile
    If fso.FileExists(paths.Item("log")) Then
      fso.DeleteFile(paths.Item("log"))
    End If
  End If    
  
  'create progress bar
  Dim sql : sql = ""
  Dim stot : stot = 0
  Dim atot : atot = 0
  Dim albums : Set albums = Nothing
  Dim prog : Set prog = SDB.Progress
  If list Is Nothing Then
    If Singles = 1 Then
      sql = "SELECT Count(*) AS SingleCount FROM Songs WHERE Album = ''"
      If Debug Then logf.WriteLine Time&" "&sql
      Set albums = SDB.Database.OpenSQL(sql)
      stot = albums.ValueByName("SingleCount")  
    End If
    sql = "SELECT Count(*) AS AlbumCount FROM Albums WHERE Albums.ID>"&StartID&" AND Albums.Album != '' "&Extra
    If Debug Then logf.WriteLine Time&" "&sql
    On Error Resume Next
    Set albums = SDB.Database.OpenSQL(sql)
    If Err.Number <> 0 Then
      Call SDB.MessageBox("Batch Art Finder - your Extra option has created an invalid SQL statement.",mtError,Array(mbOk))
      form.Common.ControlName = ""
      Set SDB.Objects("BatchArtFinderForm") = Nothing
      Err.Clear      
      Exit Sub    
    End If
    atot = albums.ValueByName("AlbumCount")
    On Error Goto 0  
    prog.MaxValue = atot+stot
  Else
    prog.MaxValue = list.Count 'this is set properly later
  End If
  prog.Value = 0
  prog.Text = "Batch Art Finder - initialising..."
  SDB.ProcessMessages
  
  'create document
  Dim doc : Set doc = fso.CreateTextFile(paths.Item("htm"),True) 
  If doc Is Nothing Then
    If fso.FileExists(paths.Item("htm")) Then
      Call fso.DeleteFile(paths.Item("htm"))
    End If
    Set doc = fso.CreateTextFile(paths.Item("htm"),True)
    If doc Is Nothing Then
      Call SDB.MessageBox("Batch Art Finder - interface could not be created.",mtInformation,Array(mbOk))
      form.Common.ControlName = ""
      Set SDB.Objects("BatchArtFinderForm") = Nothing 
      Exit Sub
    End If
  End If
  doc.WriteLine "<!DOCTYPE HTML PUBLIC ""-//W3C//DTD HTML 4.0 Transitional//EN"">" 
  doc.WriteLine "<html><head><meta http-equiv=content-type content=""text/html; charset=UTF-8"">" 
  doc.WriteLine "<title>Batch Art Finder</title></head><style type=text/css>" 
  doc.WriteLine "  body{font-family:'Verdana',sans-serif; font-size:9pt; color:#000000;}" 
  doc.WriteLine "  P{font-family:'Verdana',sans-serif; font-size:8pt; color:#000000;}" 
  doc.WriteLine "  TD{font-family:'Verdana',sans-serif; font-size:7pt; color:#000000;}"  
  doc.WriteLine "</style><script Language='VBScript'>"
  doc.WriteLine "Function reset(id)"
  doc.WriteLine "  On Error Resume Next"
  doc.WriteLine "  Set SDB = CreateObject(""SongsDB.SDBApplication"")"
  doc.WriteLine "  On Error Goto 0"
  doc.WriteLine "  If Not (SDB is Nothing) Then"
  doc.WriteLine "    Set data = SDB.Objects(""BAFData""&id)"
  doc.WriteLine "    If Not (data Is Nothing) Then"
  doc.WriteLine "      inartist = data.Item(""oartist"")"
  doc.WriteLine "      inalbum = data.Item(""oalbum"")"
  doc.WriteLine "      artist = mapname(inartist)"
  doc.WriteLine "      album = mapname(inalbum)"
  doc.WriteLine "      data.Item(""artist"") = inartist"
  doc.WriteLine "      data.Item(""album"") = inalbum"
  doc.WriteLine "      data.Item(""url"") = geturl(SDB,artist,album)"  
  doc.WriteLine "      data.Item(""image"") = ""0"""
  doc.WriteLine "      SDB.Objects(""BAFData""&id) = data"
  doc.WriteLine "    End If"
  doc.WriteLine "  End If"
  doc.WriteLine "End Function"
  doc.WriteLine "Function editquery(id)"
  doc.WriteLine "  On Error Resume Next"
  doc.WriteLine "  Set SDB = CreateObject(""SongsDB.SDBApplication"")"
  doc.WriteLine "  On Error Goto 0"
  doc.WriteLine "  If Not (SDB is Nothing) Then"
  doc.WriteLine "    Set data = SDB.Objects(""BAFData""&id)"
  doc.WriteLine "    If Not (data Is Nothing) Then"
  doc.WriteLine "      inartist = """""
  doc.WriteLine "      Source = SDB.IniFile.IntValue(""BatchArtFinder"",""Source"")"
  doc.WriteLine "      If (Source = 0) Or (Source = 4) Then"
  doc.WriteLine "        inartist = InputBox(""Artist name:"",""BatchArtFinder"",data.Item(""artist""))"
  doc.WriteLine "      End If"
  doc.WriteLine "      If inartist = """" Then"
  doc.WriteLine "        inartist = data.Item(""artist"")"
  doc.WriteLine "      End If"
  doc.WriteLine "      artist = mapname(inartist)"
  doc.WriteLine "      inalbum = InputBox(""Album name:"",""BatchArtFinder"",data.Item(""album""))"
  doc.WriteLine "      If inalbum = """" Then"
  doc.WriteLine "        inalbum = data.Item(""album"")"
  doc.WriteLine "      End If"
  doc.WriteLine "      album = mapname(inalbum)"
  doc.WriteLine "      data.Item(""artist"") = inartist"
  doc.WriteLine "      data.Item(""album"") = inalbum"
  doc.WriteLine "      data.Item(""url"") = geturl(SDB,artist,album)"
  doc.WriteLine "      data.Item(""image"") = ""0"""
  doc.WriteLine "      SDB.Objects(""BAFData""&id) = data"
  doc.WriteLine "    End If"
  doc.WriteLine "  End If"
  doc.WriteLine "End Function"
  doc.WriteLine "Function geturl(SDB,artist,album)"
  doc.WriteLine "  Select Case SDB.IniFile.IntValue(""BatchArtFinder"",""Source"")"
  doc.WriteLine "    Case 0"
  doc.WriteLine "      geturl = ""http://images.google.com/images?q=""&artist&""+""&album"
  doc.WriteLine "      Select Case SDB.IniFile.IntValue(""BatchArtFinder"",""ImageSize"")"
  doc.WriteLine "        Case 0"
  doc.WriteLine "          ImageSize = ""icon"""
  doc.WriteLine "        Case 1"
  doc.WriteLine "          ImageSize = ""medium"""
  doc.WriteLine "        Case 2"
  doc.WriteLine "          ImageSize = ""xxlarge"""
  doc.WriteLine "        Case Else"
  doc.WriteLine "          ImageSize = """""
  doc.WriteLine "      End Select"
  doc.WriteLine "      geturl = geturl&""&svnum=100&hl=en&lr=lang_en&newwindow=0&safe=off&sa=G&imgsz=""&ImageSize&""&ie=UTF-8&oe=UTF-8"""
  doc.WriteLine "    Case 1"
  doc.WriteLine "      geturl = ""http://rateyourmusic.com/search?searchterm=""&album&""&type=l"""
  doc.WriteLine "    Case 2"  
  doc.WriteLine "      geturl = ""http://www.juno.co.uk/search.php?page=1&q=""&album&""&x=0&y=0&precision=all&column=all"""
  doc.WriteLine "    Case 3"
  doc.WriteLine "      geturl = ""http://albumart.org/covers2.php?pg=1&search=""&album&""&type=music"""  
  doc.WriteLine "    Case 4"
  doc.WriteLine "      geturl = ""http://ws.audioscrobbler.com/1.0/album/""&artist&""/""&album&""/info.xml"""    
  doc.WriteLine "    Case 5"
  doc.WriteLine "      geturl = SDB.IniFile.StringValue(""BatchArtFinder"",""Locations"")"      
  '!
  doc.WriteLine "  End Select"  
  doc.WriteLine "End Function"
  doc.WriteLine "Function mapname(name)"
  doc.WriteLine "  nameUTF8 = UTF8_Encode(name)"
  doc.WriteLine "  For i = 1 To Len(nameUTF8)"
  doc.WriteLine "    Dim c : c = Asc(Mid(nameUTF8,i))"
  doc.WriteLine "    If (c>32 And c<48) Or (c>57 And c<65) Or (c>90 And c<97) Or (c>122 And c<127) Then"
  doc.WriteLine "      mapname = mapname&""%""&CStr(Hex(c))"
  doc.WriteLine "    Else"
  doc.WriteLine "      mapname = mapname&Chr(c)"
  doc.WriteLine "    End If"
  doc.WriteLine "  Next"
  doc.WriteLine "  mapname = Replace(mapname,"" "",""+"")"
  doc.WriteLine "End Function"
  doc.WriteLine "Function UTF8_Encode(ByVal UnicodeString)"
  doc.WriteLine "  For i = 1 To Len(UnicodeString)"
  doc.WriteLine "    UnicodeCharCode = AscW(Mid(UnicodeString,i))"
  doc.WriteLine "    If UnicodeCharCode < &H80 Then"
  doc.WriteLine "      UTF8_temp = Chr(UnicodeCharCode)"
  doc.WriteLine "    ElseIf UnicodeCharCode < &H800 Then"
  doc.WriteLine "      UTF8_temp = Chr(&H80 + (UnicodeCharCode And &H3F))"
  doc.WriteLine "      UnicodeCharCode = UnicodeCharCode \ &H40"
  doc.WriteLine "      UTF8_temp = Chr(&HC0 + (UnicodeCharCode And &H1F)) & UTF8_temp"
  doc.WriteLine "    Else"
  doc.WriteLine "      UTF8_temp = Chr(&H80 + (UnicodeCharCode And &H3F))"
  doc.WriteLine "      UnicodeCharCode = UnicodeCharCode \ &H40"
  doc.WriteLine "      UTF8_temp = Chr(&H80 + (UnicodeCharCode And &H3F)) & UTF8_temp"
  doc.WriteLine "      UnicodeCharCode = UnicodeCharCode \ &H40"
  doc.WriteLine "      UTF8_temp = Chr(&HE0 + (UnicodeCharCode And &HF)) & UTF8_temp"
  doc.WriteLine "    End If"
  doc.WriteLine "    UTF8_Encode = UTF8_Encode & UTF8_temp"
  doc.WriteLine "  Next"
  doc.WriteLine "End Function"
  doc.WriteLine "Function previous(id)"
  doc.WriteLine "  On Error Resume Next"
  doc.WriteLine "  Set SDB = CreateObject(""SongsDB.SDBApplication"")"
  doc.WriteLine "  On Error Goto 0"
  doc.WriteLine "  If Not (SDB is Nothing) Then"
  doc.WriteLine "    Set data = SDB.Objects(""BAFData""&id)"
  doc.WriteLine "    If Not (data Is Nothing) Then"
  doc.WriteLine "      data.Item(""previous"") = ""1"""
  doc.WriteLine "      SDB.Objects(""BAFData""&id) = data"
  doc.WriteLine "    End If"
  doc.WriteLine "  End If"
  doc.WriteLine "End Function"  
  doc.WriteLine "</script><script Language='JavaScript'>"
  doc.WriteLine "function skipimg(id){"
  doc.WriteLine "  reset(id);"
  doc.WriteLine "  document.getElementById(""img""+id).innerHTML=""[Skipped]"";"
  doc.WriteLine "  document.getElementById(""dim""+id).innerHTML="""";"
  doc.WriteLine "  document.getElementById(""sav""+id).checked = false;"
  doc.WriteLine "}"
  doc.WriteLine "function nextimg(id){"  
  doc.WriteLine "  document.getElementById(""img""+id).innerHTML=""[Queued]"";"
  doc.WriteLine "  document.getElementById(""dim""+id).innerHTML="""";"  
  doc.WriteLine "}"
  doc.WriteLine "function editimg(id){"
  doc.WriteLine "  editquery(id);"
  doc.WriteLine "  document.getElementById(""img""+id).innerHTML=""[Queued]"";"
  doc.WriteLine "  document.getElementById(""dim""+id).innerHTML="""";"
  doc.WriteLine "}"
  doc.WriteLine "function previmg(id){"    
  doc.WriteLine "  previous(id);"
  doc.WriteLine "  document.getElementById(""img""+id).innerHTML=""[Queued]"";"
  doc.WriteLine "  document.getElementById(""dim""+id).innerHTML="""";"  
  doc.WriteLine "}"  
  doc.WriteLine "function select(mode){"    
  doc.WriteLine "  var i = 1;"
  doc.WriteLine "  var d = null;"
  doc.WriteLine "  while(d=document.getElementById(""sav""+i)) {"  
  doc.WriteLine "    switch(mode) {"    
  doc.WriteLine "      case 0:"    
  doc.WriteLine "        d.checked = true;"    
  doc.WriteLine "        break;"    
  doc.WriteLine "      case 1:"    
  doc.WriteLine "        d.checked = false;"    
  doc.WriteLine "        break;"
  doc.WriteLine "      case 2:"    
  doc.WriteLine "        d.checked = !d.checked;"
  doc.WriteLine "        break;"
  doc.WriteLine "    }"    
  doc.WriteLine "    i++;"    
  doc.WriteLine "  }"    
  doc.WriteLine "}"    
  doc.WriteLine "function trynextall(){"    
  doc.WriteLine "  var i = 1;"
  doc.WriteLine "  var d = null;"
  doc.WriteLine "  while(d=document.getElementById(""img""+i)) {"
  doc.WriteLine "    d.innerHTML = ""[Queued]"";"    
  doc.WriteLine "    if(d=document.getElementById(""dim""+i)) {"    
  doc.WriteLine "      d.innerHTML = """";"
  doc.WriteLine "    }"      
  If CheckAll = 1 Then
    doc.WriteLine "    if(d=document.getElementById(""sav""+i)) {"    
    doc.WriteLine "      d.checked = true;"    
    doc.WriteLine "    }"    
  End If
  doc.WriteLine "    i++;"    
  doc.WriteLine "  }"    
  doc.WriteLine "}"      
  doc.WriteLine "</script><body><p><a href=""javascript:select(0)"">Select all</a> <a href=""javascript:select(1)"">Select none</a> <a href=""javascript:select(2)"">Select inverse</a> <a href=""javascript:trynextall()"">Try next image for all</a></p>"  
  doc.WriteLine "<table border=""1"" cellpadding=""4"" cellspacing=""0"" width=""100%"">"  
  
  'loop through all albums looking for those without art
  Dim album,id,iter,avgyear,totlength,j
  Dim i : i = 0
  If list Is Nothing Then
    If StartID > -1 Then
      If Debug Then
        logf.WriteLine Time&" (All albums)"
        If StartID > 0 Then
          logf.WriteLine Time&" (Starting from ID="&StartID&")"
        End If
      End If      
      sql = "SELECT Albums.ID FROM Albums WHERE Albums.ID>"&StartID&" AND Albums.Album != '' "&Extra&" GROUP BY Albums.ID"
      If Debug Then logf.WriteLine Time&" "&sql
      Set albums = SDB.Database.OpenSQL(sql)
      Do While Not albums.EOF
        prog.Increase
        prog.Text = "Batch Art Finder - checking album "&prog.Value&"/"&prog.MaxValue&" (found: "&i&")..."
        id = albums.ValueByName("ID")
        Dim songs : Set songs = SDB.Database.QuerySongs("AND (Songs.IDAlbum="&id&")")
        If Not (songs.EOF) Then
          Set album = songs.Item.Album
          If (album.AlbumArt Is Nothing) Or (IncludeDone) Then
            'add album ID to array
            If Debug Then logf.WriteLine Time&" AlbumID="&id
            i = i + 1
            todo.Item(i) = id
            'add album to form
            sql = "SELECT Avg(Year/10000) As AvgYear, Sum(SongLength) As TotLength FROM Songs WHERE IDAlbum="&id
            If Debug Then logf.WriteLine Time&" "&sql
            Set iter = SDB.Database.OpenSQL(sql)
            avgyear = fixyear(iter.StringByName("AvgYear"))
            totlength = gettime(iter.ValueByName("TotLength")/1000)
            doc.WriteLine "<tr id='row"&i&"'><td width=50% align=right>Album: "&UTF8_Encode(album.Name)
            doc.WriteLine "<br>Artist: "&UTF8_Encode(album.Artist.Name)
            doc.WriteLine "<br>Year: "&avgyear
            doc.WriteLine "<br>Length: "&totlength
            doc.WriteLine "<br><br><a href=""javascript:nextimg('"&i&"')"">Try next image</a> "
            doc.WriteLine "<a href=""javascript:skipimg('"&i&"')"">Skip this album</a>"
            doc.WriteLine "<a href=""javascript:editimg('"&i&"')"">Edit album query</a>"
            doc.WriteLine "<a href=""javascript:previmg('"&i&"')"">Previous image</a>"
            doc.WriteLine "<p>Save:<input type=checkbox id='sav"&i&"'"&chk&"></p><div id='dim"&i&"'></div></td>"
            doc.WriteLine "<td width=50% align=left><div id='img"&i&"'>[Queued]</div></td></tr>"
          End If
        End If
        albums.Next
        'allow user to cancel
        SDB.ProcessMessages
        If prog.Terminate Then
          If Debug Then 
            logf.WriteLine Time&" (Cancelled by user)"
            logf.Close
          End If
          form.Common.ControlName = ""
          Set SDB.Objects("BatchArtFinderForm") = Nothing
          If Debug Then logf.Close
          doc.Close
          If fso.FileExists(paths.Item("htm")) Then
            Call fso.DeleteFile(paths.Item("htm"))
          End If          
          Exit Sub
        End If
        'maximum batch size
        If Maximum > 0 Then
          If i >= Maximum Then
            If Debug Then logf.WriteLine Time&" (Maximum of "&Maximum&" reached)"
            Exit Do
          End If
        End If
      Loop
    End If
    If (Singles = 1) And ((Maximum = 0) Or (i < Maximum)) Then 
      sql = ""
      If Debug Then logf.WriteLine Time&" (All singles)"
      If StartID < 0 Then
        StartID = Abs(StartID)
        sql = " AND Songs.ID>"&StartID
        If Debug Then logf.WriteLine Time&" (Starting from ID="&StartID&")"
      End If
      sql = "SELECT Songs.ID FROM Songs WHERE Songs.Album = ''"&sql&" GROUP BY Songs.ID"
      If Debug Then logf.WriteLine Time&" "&sql
      Set albums = SDB.Database.OpenSQL(sql)
      Do While Not albums.EOF
        prog.Increase
        prog.Text = "Batch Art Finder - checking album "&prog.Value&"/"&prog.MaxValue&" (found: "&i&")..."
        id = albums.ValueByName("ID")
        Set songs = SDB.Database.QuerySongs("AND (Songs.ID="&id&")")
        If Not (songs.EOF) Then
          Set album = songs.Item
          If (album.AlbumArt.Count = 0) Or (IncludeDone) Then
            'add song ID to array
            If Debug Then logf.WriteLine Time&" SongID="&id
            i = i + 1
            todo.Item(i) = "*"&id
            'add album to form
            doc.WriteLine "<tr id='row"&i&"'><td width=50% align=right>Single: "&UTF8_Encode(album.Title)
            doc.WriteLine "<br>Artist: "&UTF8_Encode(album.Artist.Name)
            doc.WriteLine "<br>Year: "&fixyear(album.Year)
            doc.WriteLine "<br>Length: "&gettime(album.SongLength/1000)
            doc.WriteLine "<br><br><a href=""javascript:nextimg('"&i&"')"">Try next image</a> "
            doc.WriteLine "<a href=""javascript:skipimg('"&i&"')"">Skip this album</a>"
            doc.WriteLine "<a href=""javascript:editimg('"&i&"')"">Edit album query</a>"
            doc.WriteLine "<a href=""javascript:previmg('"&i&"')"">Previous image</a>"
            doc.WriteLine "<p>Save:<input type=checkbox id='sav"&i&"'"&chk&"></p><div id='dim"&i&"'></div></td>"
            doc.WriteLine "<td width=50% align=left><div id='img"&i&"'>[Queued]</div></td></tr>"
          End If
        End If
        albums.Next
        'allow user to cancel
        SDB.ProcessMessages
        If prog.Terminate Then
          If Debug Then 
            logf.WriteLine Time&" (Cancelled by user)"
            logf.Close
          End If
          form.Common.ControlName = ""
          Set SDB.Objects("BatchArtFinderForm") = Nothing
          If Debug Then logf.Close
          doc.Close
          If fso.FileExists(paths.Item("htm")) Then
            Call fso.DeleteFile(paths.Item("htm"))
          End If            
          Exit Sub
        End If
        'maximum batch size
        If Maximum > 0 Then
          If i >= Maximum Then
            If Debug Then logf.WriteLine Time&" (Maximum of "&Maximum&" reached)"
            Exit Do
          End If
        End If
      Loop    
    End If
  Else
    If Debug Then logf.WriteLine Time&" (Selected albums)"
    prog.MaxValue = total + zero
    Set albums = list.Albums
    For j = 0 To albums.Count-1 
      prog.Increase
      prog.Text = "Batch Art Finder - checking album "&prog.Value&"/"&prog.MaxValue&" (found: "&i&")..."
      Set album = albums.Item(j)
      id = Album.ID
      If id > 0 Then
        If (album.AlbumArt Is Nothing) Or (IncludeDone) Then
          'add album ID to array
          If Debug Then logf.WriteLine Time&" AlbumID="&id
          i = i + 1
          todo.Item(i) = id
          'add album to form
          sql = "SELECT Avg(Year/10000) As AvgYear, Sum(SongLength) As TotLength FROM Songs WHERE IDAlbum="&id
          If Debug Then logf.WriteLine Time&" "&sql
          Set iter = SDB.Database.OpenSQL(sql)
          avgyear = fixyear(iter.StringByName("AvgYear"))
          totlength = gettime(iter.ValueByName("TotLength")/1000)
          doc.WriteLine "<tr id='row"&i&"'><td width=50% align=right>Album: "&UTF8_Encode(album.Name)
          doc.WriteLine "<br>Artist: "&UTF8_Encode(album.Artist.Name)
          doc.WriteLine "<br>Year: "&avgyear
          doc.WriteLine "<br>Length: "&totlength
          doc.WriteLine "<br><br><a href=""javascript:nextimg('"&i&"')"">Try next image</a> "
          doc.WriteLine "<a href=""javascript:skipimg('"&i&"')"">Skip this album</a>"
          doc.WriteLine "<a href=""javascript:editimg('"&i&"')"">Edit album query</a>"
          doc.WriteLine "<a href=""javascript:previmg('"&i&"')"">Previous image</a>"
          doc.WriteLine "<p>Save:<input type=checkbox id='sav"&i&"'"&chk&"></p><div id='dim"&i&"'></div></td>"
          doc.WriteLine "<td width=50% align=left><div id='img"&i&"'>[Queued]</div></td></tr>"
        End If
      End If
      'allow user to cancel
      SDB.ProcessMessages
      If prog.Terminate Then
        If Debug Then 
          logf.WriteLine Time&" (Cancelled by user)"
          logf.Close
        End If
        form.Common.ControlName = ""
        Set SDB.Objects("BatchArtFinderForm") = Nothing
        If Debug Then logf.Close
        doc.Close
        If fso.FileExists(paths.Item("htm")) Then
          Call fso.DeleteFile(paths.Item("htm"))
        End If          
        Exit Sub
      End If
      'maximum batch size
      If Maximum > 0 Then
        If i >= Maximum Then
          If Debug Then logf.WriteLine Time&" (Maximum of "&Maximum&" reached)"
          Exit For
        End If
      End If      
    Next  
    If (Singles = 1) And ((Maximum = 0) Or (i < Maximum)) Then
      If Debug Then logf.WriteLine Time&" (Selected singles)"
      For j = 0 To list2.Count-1 
        prog.Increase
        prog.Text = "Batch Art Finder - checking album "&prog.Value&"/"&prog.MaxValue&" (found: "&i&")..."
        Set album = list2.Item(j)
        id = Album.ID
        If id > 0 Then
          If (album.AlbumArt.Count = 0) Or (IncludeDone) Then
            'add album ID to array
            If Debug Then logf.WriteLine Time&" SongID="&id
            i = i + 1
            todo.Item(i) = "*"&id
            'add single to form
            doc.WriteLine "<tr id='row"&i&"'><td width=50% align=right>Single: "&UTF8_Encode(album.Title)
            doc.WriteLine "<br>Artist: "&UTF8_Encode(album.Artist.Name)
            doc.WriteLine "<br>Year: "&fixyear(album.Year)
            doc.WriteLine "<br>Length: "&gettime(album.SongLength/1000)
            doc.WriteLine "<br><br><a href=""javascript:nextimg('"&i&"')"">Try next image</a> "
            doc.WriteLine "<a href=""javascript:skipimg('"&i&"')"">Skip this album</a>"
            doc.WriteLine "<a href=""javascript:editimg('"&i&"')"">Edit album query</a>"
            doc.WriteLine "<a href=""javascript:previmg('"&i&"')"">Previous image</a>"
            doc.WriteLine "<p>Save:<input type=checkbox id='sav"&i&"'"&chk&"></p><div id='dim"&i&"'></div></td>"
            doc.WriteLine "<td width=50% align=left><div id='img"&i&"'>[Queued]</div></td></tr>"
          End If 
        End If
        'allow user to cancel
        SDB.ProcessMessages
        If prog.Terminate Then
          If Debug Then 
            logf.WriteLine Time&" (Cancelled by user)"
            logf.Close
          End If
          form.Common.ControlName = ""
          Set SDB.Objects("BatchArtFinderForm") = Nothing
          If Debug Then logf.Close
          doc.Close
          If fso.FileExists(paths.Item("htm")) Then
            Call fso.DeleteFile(paths.Item("htm"))
          End If            
          Exit Sub
        End If
        'maximum batch size
        If Maximum > 0 Then
          If i >= Maximum Then
            If Debug Then logf.WriteLine Time&" (Maximum of "&Maximum&" reached)"
            Exit For
          End If
        End If      
      Next    
    End If
  End If
  
  'finish document and show form
  Set prog = Nothing
  If Debug Then logf.WriteLine Time&" Total="&i
  doc.WriteLine "</table><p><a href=""javascript:select(0)"">Select all</a> <a href=""javascript:select(1)"">Select none</a> <a href=""javascript:select(2)"">Select inverse</a> <a href=""javascript:trynextall()"">Try next image for all</a></p>"
  If SDB.IniFile.BoolValue("BatchArtFinder","SetSize") Then
    doc.WriteLine "<span style=""visibility:hidden""><img id=""img0"" src=""""></span>"
  End If  
  doc.WriteLine "</body></html>" 
  doc.Close
  If i > 0 Then
    'show form
    Call wb.Interf.Navigate(paths.Item("htm"))
    form.Common.Visible = True
    If list Is Nothing Then
      btn3.Common.Left = foot.Common.Width - btn3.Common.Width
      btn2.Common.Left = btn3.Common.Left - btn2.Common.Width
    Else
      btn2.Common.Left = foot.Common.Width - btn2.Common.Width
    End If
    btn1.Common.Left = btn2.Common.Left - btn1.Common.Width
    Set SDB.Objects("BatchArtFinderForm") = form
    Set SDB.Objects("BatchArtFinderDoc") = wb.Interf.Document
  
    'start first search
    todo.Item("key") = 0
    todo.Item("max") = i
    todo.Item("mid") = 0
    Set SDB.Objects("BatchArtFinderToDo") = todo
    Call TriggerNext()
  Else    
    form.Common.ControlName = ""
    Set SDB.Objects("BatchArtFinderForm") = Nothing
    If fso.FileExists(paths.Item("htm")) Then
      Call fso.DeleteFile(paths.Item("htm"))
    End If 
    ini.IntValue("BatchArtFinder","StartID") = 0  
    If IncludeDone Then
      Call SDB.MessageBox("Batch Art Finder - all albums have been processed.",mtInformation,Array(mbOk))
    Else
      If rep.Exists("include") Then
        i = Int(rep.Item("include"))
        If i = mrNo Then
          Call SDB.MessageBox("Batch Art Finder - there are no albums with missing artwork.",mtInformation,Array(mbOk))
        End If
      Else
        i = SDB.MessageBox("Batch Art Finder - there are no albums with missing artwork, do you want to process albums with artwork?",mtConfirmation,Array(mbYes,mbNo))
        rep.Item("include") = i
        If Persist = 1 Then
          ini.StringValue("BatchArtFinder","Replies3") = i
          hnt = GetHint()
          If Not (hnt = "") Then
            hnt = Chr(13)&"-"&Chr(13)&hnt
          End If
          SDB.Objects("BAFToolbarButton").Caption = "Batch Art Finder"&hnt          
        End If
      End If
      If i = mrYes Then
        If Debug Then logf.Close 
        ini.BoolValue("BatchArtFinder","IncludeDone") = True
        Call DoBatchArtFinder(itm)
        Exit Sub
      End If
    End If
  End If
  If Debug Then logf.Close 
End Sub 

Sub FormClose(form)
  If Not (form Is Nothing) Then
    Dim ini : Set ini = SDB.IniFile
    If ini.IntValue("BatchArtFinder","PanelType") = 3 Then
      ini.IntValue("BatchArtFinder","PanelDockedTo") = form.DockedTo
    Else
      ini.IntValue("BatchArtFinder","PanelTop") = form.Common.Top
      ini.IntValue("BatchArtFinder","PanelLeft") = form.Common.Left
    End If
    ini.IntValue("BatchArtFinder","PanelWidth") = form.Common.Width
    ini.IntValue("BatchArtFinder","PanelHeight") = form.Common.Height
    form.Common.ControlName = ""
  End If
  Call ClearData
End Sub

Sub TriggerNext()
  'check for to-do list
  SDB.ProcessMessages
  Dim todo : Set todo = SDB.Objects("BatchArtFinderToDo")
  If todo Is Nothing Then 
    Exit Sub
  End If
  If Int(todo.Item("mid")) = 1 Then
    Exit Sub
  End If
  
  'get next item from list
  Dim key : key = Int(todo.Item("key"))+1
  Dim max : max = Int(todo.Item("max"))
  todo.Item("key") = key
  todo.Item("mid") = 1

  'trigger next search
  Dim form : Set form = SDB.Objects("BatchArtFinderForm")
  If key <= max Then  
    Dim doc : Set doc = SDB.Objects("BatchArtFinderDoc")
    If Not (doc Is Nothing) Then
      'update status message 
      Dim i,div,qc,sc,nc,cc
      For i = 1 To max
        Set div = doc.getElementById("img"&i)
        If Not (div Is Nothing) Then
          If div.innerHTML = "[Queued]" Then 
            qc = qc + 1
          End If  
          If div.innerHTML = "[Skipped]" Then 
            sc = sc + 1
          End If
          If div.innerHTML = "[No results]" Then 
            nc = nc + 1
          End If
          If div.innerHTML = "[Searching]" Then 
            cc = cc + 1
          End If                  
        End If    
      Next 
      Dim fc : fc = max - (qc+sc+nc+cc)
      Dim lbl : Set lbl = form.Common.ChildControl("lbl1")
      lbl.Caption = "Found:"&fc&"/"&max&" (Queued:"&Int(qc)&" - Skipped:"&Int(sc)&" - No results:"&Int(nc)&")"
      
      'check item status        
      Set div = doc.getElementById("img"&key)
      If Not (div Is Nothing) Then
        If div.innerHTML = "[Queued]" Then
          div.innerHTML = "[Searching]"
          Dim id : id = todo.Item(key)
          Dim songs : Set songs = Nothing
          If IsNumeric(id) Then
            Set songs = SDB.Database.QuerySongs("AND (Songs.IDAlbum="&id&")")
            If Not (songs.EOF) Then          
              If songs.Item.Album.Tracks.Count > 0 Then
                Call ArtFinder(songs.Item.Album)
                todo.Item("mid") = 0
                Exit Sub
              Else
                div.innerHTML = "[Skipped]"
              End If
            End If         
          Else
            Set songs = SDB.Database.QuerySongs("AND (Songs.ID="&Mid(id,2)&")")
            If Not (songs.EOF) Then     
              Call ArtFinderSingle(songs.Item)
              todo.Item("mid") = 0
              Exit Sub
            End If          
          End If
        Else
          Dim div2 : Set div2 = doc.getElementById("dim"&key)
          If Not (div2 Is Nothing) Then
            If div2.innerHTML = "" Then
              Dim img : Set img = div.Children.Item(0)
              If Not (img Is Nothing) Then
                Set img = img.Children.Item(0)
                If Not (img Is Nothing) Then
                  Dim ini : Set ini = SDB.IniFile
                  MinW = ini.IntValue("BatchArtFinder","MinW")
                  MaxW = ini.IntValue("BatchArtFinder","MaxW")
                  MinH = ini.IntValue("BatchArtFinder","MinH")
                  MaxH = ini.IntValue("BatchArtFinder","MaxH")        
                  If ini.BoolValue("BatchArtFinder","SetSize") Then
                    Dim img2 : Set img2 = doc.getElementById("img0")
                    If Not (img2 Is Nothing) Then
                      img2.src = img.src
                      If (img2.Width > 0) And (img2.Height > 0) Then
                        If (img2.Width = 28) And (img2.Height = 30) Then 'red cross
                          div.innerHTML = "[Queued]"              
                        Else
                          If (img2.Width<MinW) Or (img2.Width>MaxW) Or (img2.Height<MinH) Or (img2.Height>MaxH) Then                   
                            div.innerHTML = "[Queued]"              
                          Else
                            div2.innerHTML = "Size: "&img2.Width&"x"&img2.Height         
                          End If
                        End If
                      End If
                      img2.src = ""
                    End If
                  Else
                    If (img.Width > 0) And (img.Height > 0) Then
                      If (img.Width = 28) And (img.Height = 30) Then 'red cross
                        div.innerHTML = "[Queued]"              
                      Else                    
                        If (img.Width<MinW) Or (img.Width>MaxW) Or (img.Height<MinH) Or (img.Height>MaxH) Then                   
                          div.innerHTML = "[Queued]"              
                        Else
                          div2.innerHTML = "Size: "&img.Width&"x"&img.Height         
                        End If
                      End If
                    End If
                  End If
                End If
              End If
            End If
          End If
        End If
      End If
    End If
    todo.Item("mid") = 0
    Call TriggerNext()
  Else
    'check form still exists and visible
    If Not (form Is Nothing) Then
      If form.Common.Visible Then
        'pause then start from the top
        todo.Item("key") = 0
        todo.Item("mid") = 0
        Dim Tmr : Set Tmr = SDB.CreateTimer(1000)
        Script.RegisterEvent Tmr, "OnTimer", "DelayTrigger"
        Exit Sub
      End If 
    End If
    Call ClearData
  End If
End Sub 

Sub DelayTrigger(DelayTimer)
  Script.UnregisterEvents DelayTimer
  Dim todo : Set todo = SDB.Objects("BatchArtFinderToDo")
  If todo Is Nothing Then 
    Exit Sub
  End If  
  If Int(todo.Item("mid")) = 0 Then
    Call TriggerNext()
  End If
End Sub

Sub ArtFinder(itm)
  'check for previous search
  Dim url : url = ""
  Dim todo : Set todo = SDB.Objects("BatchArtFinderToDo")
  Dim data : Set data = SDB.Objects("BAFData"&todo.Item("key"))
  Source = SDB.IniFile.IntValue("BatchArtFinder","Source")
  If Not (data Is Nothing) Then
    'use previous data
    url = data.Item("url")
  Else
    'create search url
    Dim path : path = itm.Tracks.Item(0).Path
    Dim artist : artist = mapname(itm.Artist.Name)
    Dim album : album = mapname(itm.Name)
    Select Case Source
      Case 0
        url = "http://images.google.com/images?q="&artist&"+"&album
        Select Case SDB.IniFile.IntValue("BatchArtFinder","ImageSize")
          Case 0 
            ImageSize = "icon"
          Case 1 
            ImageSize = "medium"
          Case 2
            ImageSize = "xxlarge"
          Case Else
            ImageSize = ""
        End Select
        url = url&"&svnum=100&hl=en&lr=lang_en&newwindow=0&safe=off&sa=G&imgsz="&ImageSize&"&ie=UTF-8&oe=UTF-8"
      Case 1
        url = "http://rateyourmusic.com/search?searchterm="&album&"&type=l"
      Case 2
        url = "http://www.juno.co.uk/search.php?page=1&q="&album&"&x=0&y=0&precision=all&column=all"
      Case 3
        url = "http://albumart.org/covers2.php?pg=1&search="&album&"&type=music"
      Case 4
        url = "http://ws.audioscrobbler.com/1.0/album/"&artist&"/"&album&"/info.xml"            
      Case 5
        url = SDB.IniFile.StringValue("BatchArtFinder","Locations")
      '!
    End Select
    
    'store data and url
    Set data = CreateObject("Scripting.Dictionary")
    Dim i : i = InStrRev(path,"\")-1
    If i > 0 Then
      data.Add "path",Left(path,i)
    Else
      Call out("path="&path)
      data.Add "path",""
    End If
    data.Add "artist",itm.Artist.Name
    data.Add "album",itm.Name
    data.Add "oartist",itm.Artist.Name
    data.Add "oalbum",itm.Name    
    data.Add "id",itm.ID
    data.Add "image","0"
    data.Add "url",url
    i = InStrRev(path,".")+1
    If i > 2 Then
      data.Add "ext",UCase(Mid(path,i+1))
    Else
      Call out("path="&path)
      data.Add "ext",""
    End If
    data.Add "previous","0"
    Set SDB.Objects("BAFData"&todo.Item("key")) = data
  End If
  
  'navigate to search url
  Dim Tmr : Set Tmr = SDB.CreateTimer(500)
  Select Case Source
    Case 4
      Dim xml : Set xml = CreateObject("Microsoft.XMLDOM")
      xml.async = True
      Call xml.Load(url)
      Set SDB.Objects("BatchArtFinderXML") = xml
      Script.RegisterEvent Tmr, "OnTimer", "WaitTilReady2"
    Case 5
      Call LocalHost(todo.Item("key"))
    Case Else  
      Dim IE : Set IE = SDB.Objects("BatchArtFinderShell")
      If Not (IE Is Nothing) Then
        IE.Navigate url
        Script.RegisterEvent Tmr, "OnTimer", "WaitTilReady"
      End If
  End Select
End Sub

Sub ArtFinderSingle(itm)
  'check for previous search
  Dim url : url = ""
  Dim todo : Set todo = SDB.Objects("BatchArtFinderToDo")
  Dim data : Set data = SDB.Objects("BAFData"&todo.Item("key"))
  Source = SDB.IniFile.IntValue("BatchArtFinder","Source")
  If Not (data Is Nothing) Then
    'use previous data
    url = data.Item("url")
  Else
    'create search url
    Dim path : path = itm.Path
    Dim i : i = InStrRev(path,"\")-1
    Dim artist : artist = mapname(itm.Artist.Name)
    Dim album : album = mapname(itm.Title)
    Select Case Source
      Case 0
        url = "http://images.google.com/images?q="&artist 
        If SDB.IniFile.IntValue("BatchArtFinder","Include") = 1 Then
          url = url&"+"&album
        End If
        Select Case SDB.IniFile.IntValue("BatchArtFinder","ImageSize")
          Case 0 
            ImageSize = "icon"
          Case 1 
            ImageSize = "medium"
          Case 2
            ImageSize = "xxlarge"
          Case Else
            ImageSize = ""
        End Select
        url = url&"&svnum=100&hl=en&lr=lang_en&newwindow=0&safe=off&sa=G&imgsz="&ImageSize&"&ie=UTF-8&oe=UTF-8"
      Case 1
        url = "http://rateyourmusic.com/search?searchterm="&album&"&type=l"
      Case 2
        url = "http://www.juno.co.uk/search.php?page=1&q="&album&"&x=0&y=0&precision=all&column=all"        
      Case 3
        url = "http://albumart.org/covers2.php?pg=1&search="&album&"&type=music"        
      Case 4
        url = "http://ws.audioscrobbler.com/1.0/album/"&artist&"/"&album&"/info.xml"             
      Case 5
        url = SDB.IniFile.StringValue("BatchArtFinder","Locations")
      '!
    End Select
    
    'store data and url
    Set data = CreateObject("Scripting.Dictionary")
    data.Add "path",Mid(path,1,i)
    data.Add "artist",itm.Artist.Name
    data.Add "album",itm.Title
    data.Add "oartist",itm.Artist.Name
    data.Add "oalbum",itm.Title    
    data.Add "id",itm.ID
    data.Add "image","0"
    data.Add "url",url
    data.Add "previous","0"
    Set SDB.Objects("BAFData"&todo.Item("key")) = data
  End If
  
  'navigate to search url
  Dim Tmr : Set Tmr = SDB.CreateTimer(500)
  Select Case Source
    Case 4
      Dim xml : Set xml = CreateObject("Microsoft.XMLDOM")
      xml.async = True
      Call xml.Load(url)
      Set SDB.Objects("BatchArtFinderXML") = xml
      Script.RegisterEvent Tmr, "OnTimer", "WaitTilReady2"
    Case 5
      Call LocalHost(todo.Item("key"))
    Case Else  
      Dim IE : Set IE = SDB.Objects("BatchArtFinderShell")
      If Not (IE Is Nothing) Then
        IE.Navigate url
        Script.RegisterEvent Tmr, "OnTimer", "WaitTilReady"
      End If
  End Select
End Sub

Sub WaitTilReady(WaitTimer)
  'check search page exists
  Dim IE : Set IE = SDB.Objects("BatchArtFinderShell")
  If (IE Is Nothing) Then
    'user cancelled
    Script.UnregisterEvents WaitTimer
  Else
    'check page is ready
    If (IE.ReadyState = 4) Then
      Script.UnregisterEvents WaitTimer
      ShowResult(IE.Document.Images)
    End If
  End If
End Sub

Function ShowResult(list)
  'get relevant data
  Dim todo : Set todo = SDB.Objects("BatchArtFinderToDo")
  If todo Is Nothing Then
    Exit Function
  End If    
  Dim data : Set data = SDB.Objects("BAFData"&todo.Item("key"))
  If data Is Nothing Then
    Exit Function
  End If    
  Dim ini : Set ini = SDB.IniFile
  Dim img : img = Int(data.Item("image"))
  Dim mess : mess = "[No results]"
  Dim i : i = 0  
  Dim str : str = ""
  Dim ext : ext = ""
  ShowResult = False
  Source = ini.IntValue("BatchArtFinder","Source")
  ImageSize = ini.IntValue("BatchArtFinder","ImageSize")
  SetSize = ini.BoolValue("BatchArtFinder","SetSize")
  If SetSize Then
    SetW = ini.IntValue("BatchArtFinder","SetW")
    SetH = ini.IntValue("BatchArtFinder","SetH")
    str = " width='"&SetW&"px' height='"&SetH&"px'"
  End If
  FileTypes = ini.StringValue("BatchArtFinder","FileTypes")
  Dim arr : arr = Split(UCase(FileTypes),",")
  Dim a : a = img
  Dim b : b = list.Length-1
  Dim c : c = 1
  If data.Item("previous") = "1" Then
    a = img-2
    b = 0
    c = -1
    data.Item("previous") = "0"
  End If
  
  'check list of images returned
  For i = a To b Step c
    Dim src : src = list.Item(i).Src    
    If InFileTypes(arr,src,ext) Then
      'update data
      src = "http:"&Mid(src,InStrRev(src,":")+1)
      src = Replace(src,"%25","%")
      src = Replace(src,"%20"," ")
      Select Case Source
        Case 1
          src = Replace(src,"/album_images/s","/album_images/")
        Case 2
          Select Case ImageSize
            Case 1
              src = Replace(src,"/75/","/150/")
              src = Replace(src,"-TN","")          
            Case 2           
              src = Replace(src,"/75/","/full/")
              src = Replace(src,"-TN","-BIG")
            Case 3           
              src = Replace(src,"/75/","/full/")
              src = Replace(src,"-TN","-BIG")              
          End Select
        '!
      End Select 
      data.Item("image") = i+1
      data.Item("source") = src
      data.Item("type") = ext
      Set SDB.Objects("BAFData"&todo.Item("key")) = data
      mess = "<a href='"&data.Item("url")&"' target='_blank'><img src='"&src&"' title='"&src&"'"&str&"></a>"
      ShowResult = True
      Exit For
    End If
  Next
 
  'update document
  Dim doc : Set doc = SDB.Objects("BatchArtFinderDoc")
  If Not (doc Is Nothing) Then
    Dim div : Set div = doc.getElementById("img"&todo.Item("key"))
    If Not (div Is Nothing) Then
      div.innerHTML = mess
    End If
  End If 
  
  'start next search
  Call TriggerNext()
End Function

Sub WaitTilReady2(WaitTimer)
  'check xml exists
  Dim xml : Set xml = SDB.Objects("BatchArtFinderXML")
  If (xml Is Nothing) Then
    'user cancelled
    Script.UnregisterEvents WaitTimer
  Else
    'check xml is ready
    If (xml.ReadyState = 4) Then
      Script.UnregisterEvents WaitTimer
      ShowResult2(xml)
    End If
  End If
End Sub

Function ShowResult2(xml)
  'get relevant data
  Dim todo : Set todo = SDB.Objects("BatchArtFinderToDo")
  If todo Is Nothing Then
    Exit Function
  End If    
  Dim data : Set data = SDB.Objects("BAFData"&todo.Item("key"))
  If data Is Nothing Then
    Exit Function
  End If 
  Dim img : img = Int(data.Item("image"))
  Dim mess : mess = "[No results]"
  ShowResult2 = False
  If img = 0 Then
    ImageSize = SDB.IniFile.IntValue("BatchArtFinder","ImageSize")
    Dim tag : tag = "large"
    Select Case ImageSize
      Case 0 
        tag = "small"
      Case 1
        tag = "medium"
    End Select
    Dim ele : Set ele = xml.getElementsByTagName(tag).Item(0)
    If Not (ele Is Nothing) Then
      Dim src : src = ele.Text
      If Not (src = "") Then
        data.Item("image") = 1
        data.Item("source") = src
        Set SDB.Objects("BAFData"&todo.Item("key")) = data
        mess = "<a href='"&data.Item("url")&"' target='_blank'><img src='"&src&"' title='"&src&"'></a>"
        ShowResult2 = True
      End If
    End If
  End If
 
  'update document
  Dim doc : Set doc = SDB.Objects("BatchArtFinderDoc")
  If Not (doc Is Nothing) Then
    Dim div : Set div = doc.getElementById("img"&todo.Item("key"))
    If Not (div Is Nothing) Then
      div.innerHTML = mess
    End If
  End If 
  
  'start next search
  Call TriggerNext()
End Function

Sub TryClick()
  CheckAll = SDB.IniFile.IntValue("BatchArtFinder","CheckAll")
  Dim doc : Set doc = SDB.Objects("BatchArtFinderDoc")
  Dim todo : Set todo = SDB.Objects("BatchArtFinderToDo")   
  Dim max : max = Int(todo.Item("max"))
  Dim i,tot,div,row,img,chk
  For i = 1 To max
    Set div = doc.getElementById("dim"&i)
    If Not (div Is Nothing) Then
      If div.innerHTML = "Saved!" Then
        Set row = doc.getElementById("row"&i)
        If Not (row Is Nothing) Then
          row.style.display = "none"
          tot = tot+1
        End If     
      Else
        div.innerHTML = ""
        Set img = doc.getElementById("img"&i)
        If Not (img Is Nothing) Then
          img.innerHTML = "[Queued]"
        End If     
        If CheckAll = 1 Then
          Set chk = doc.getElementById("sav"&i)
          If Not (chk Is Nothing) Then
            chk.checked = True
          End If
        End If
      End If
    End If    
  Next
  If tot < max Then
    Dim but : Set but = SDB.Objects("BAFClickedBtn")
    but.Common.Enabled = True
    Set SDB.Objects("BAFClickedBtn") = Nothing
  Else
    Call NextClick(Nothing)
  End If
End Sub

Sub SaveClick(ClickedBtn)
  ClickedBtn.Common.Enabled = False  
  Set SDB.Objects("BAFClickedBtn") = ClickedBtn

  'check status of images
  Dim doc : Set doc = SDB.Objects("BatchArtFinderDoc")
  Dim todo : Set todo = SDB.Objects("BatchArtFinderToDo")   
  Dim max : max = Int(todo.Item("max"))
  Dim count : count = 0
  Dim i,div
  For i = 1 To max
    Set div = doc.getElementById("sav"&i)
    If Not (div Is Nothing) Then
      If div.Checked Then
        count = count + 1
      End If     
    End If    
  Next
  
  Dim oldmax : oldmax = todo.Item("max")
  todo.Item("max") = "0"
  
  'exit if no images to save
  If count = 0 Then
    todo.Item("key") = 0
    todo.Item("max") = oldmax
    'perform save action
    Select Case SDB.IniFile.IntValue("BatchArtFinder","SaveMode")
      Case 1 'next
        Call NextClick(Nothing)
      Case 2 'exit
        Dim form : Set form = SDB.Objects("BatchArtFinderForm")
        form.Common.Visible = False
        Call FormClose(form)      
      Case 3 'trynext
        Call TryClick()
      Case Else 'nothing
        ClickedBtn.Common.Enabled = True
        Call TriggerNext()
    End Select    
    Exit Sub 
  End If
  
  'setup internet connection
  On Error Resume Next
  Dim inet : Set inet = CreateObject("InetCtls.Inet")
  If Err.Number <> 0 Then
    Dim wsh : Set wsh = CreateObject("WScript.Shell")
    wsh.RegWrite "HKLM\SOFTWARE\Classes\Licenses\78E1BDD1-9941-11cf-9756-00AA00C00908\","yjrjvqkjlqqjnqkjvprqsjnjvkuknjpjtoun","REG_SZ"  
    Dim cmd : cmd = SDB.ApplicationPath&"Scripts\Auto\msinet.ocx"
    cmd = "regsvr32 "&Chr(34)&cmd&Chr(34)&" /s"
    i = wsh.Run(cmd,1,1)
    Err.Clear
    Set inet = CreateObject("InetCtls.Inet")
    If Err.Number <> 0 Then
      Call SDB.MessageBox("Batch Art Finder - the MSInet component could not be found, please install before running this script.",mtError,Array(mbOk))
      Call ClearData
      Exit Sub
    End If    
  End If  
  On Error GoTo 0
  Set SDB.Objects("BAFinet") = Nothing
  
  'setup download queue
  Dim queue : Set queue = CreateObject("Scripting.Dictionary")
  queue.Item("i") = 1
  queue.Item("max") = max
  queue.Item("old") = oldmax
  Set SDB.Objects("BAFQueue") = queue
  
  'start first download
  If SDB.IniFile.IntValue("BatchArtFinder","Source") = 5 Then
    Call preLocalHost()
  Else
    Script.RegisterEvent SDB.CreateTimer(100), "onTimer", "preDownload"
  End If
End Sub

Sub preDownload(preTmr)
  'wait for space
  Dim inet : Set inet = SDB.Objects("BAFinet")
  If inet Is Nothing Then
    'create internet connection
    Set inet = CreateObject("InetCtls.Inet")
    Set SDB.Objects("BAFinet") = inet
    'get queue data
    Dim queue : Set queue = SDB.Objects("BAFQueue")
    Dim i : i = Int(queue.Item("i"))
    Dim max : max = Int(queue.Item("max"))
    If i > max Then 
      Script.UnregisterEvents preTmr
      Call queueIsEmpty()
      Exit Sub
    End If
    If Debug Then Call out("Initialising "&i&" of "&max)
    'check there is an image to save
    Dim doc : Set doc = SDB.Objects("BatchArtFinderDoc")
    If Not (doc Is Nothing) Then
      Dim div : Set div = doc.getElementById("sav"&i)
      If Not (div Is Nothing) Then
        If div.Checked = True Then
          Set div = doc.getElementById("img"&i)
          If Not (div Is Nothing) Then
            Dim data : Set data = SDB.Objects("BAFData"&i)
            If Left(div.innerHTML,1) = "<" Then
              'build path from mask and data
              FileMask = SDB.IniFile.StringValue("BatchArtFinder","FileMask")
              Dim dest : dest = FileMask&data.Item("type")
              dest = Replace(dest,"<path>",data.Item("path"))
              dest = Replace(dest,"<artist>",data.Item("oartist"))
              dest = Replace(dest,"<album>",data.Item("oalbum"))
              Dim path : path = Mid(dest,3)
              path = SDB.Tools.FileSystem.CorrectFilename(path)
              data.Item("dest") = Left(dest,2)&path
              Set SDB.Objects("BAFData"&i) = data
              'start next download
              inet.url = data.Item("source")
              If Debug Then Call out("Downloading "&data.Item("source"))
              inet.requestTimeout = 10
              On Error Resume Next
              Call inet.Execute
              If Err.Number > 0 Then
                If Debug Then 
                  Call out("***"&Err.Number&" - "&Err.Description&" ("&Err.Source&")")
                  Call out("Retrying...")
                End If
                Err.Clear
                Set SDB.Objects("BAFinet") = Nothing
                Exit Sub
              End If
              On Error GoTo 0
              Script.RegisterEvent SDB.CreateTimer(100), "onTimer", "theDownload"
              Exit Sub
            Else
              If SDB.IniFile.IntValue("BatchArtFinder","Default") = 1 Then
                data.Item("dest") = getnonejpg()
                Set SDB.Objects("BAFData"&i) = data
                Call postDownload(i)
              End If
            End If
          End If
        End If
      End If
    End If
    'update queue
    queue.Item("i") = i+1
    Set SDB.Objects("BAFinet") = Nothing
  End If
End Sub

Sub theDownload(theTmr)  
  'wait for connection
  Dim inet : Set inet = SDB.Objects("BAFinet")
  If Not inet.StillExecuting Then
    Script.UnregisterEvents theTmr
    If Debug Then Call out("Downloading complete")
    'get queue data
    Dim queue : Set queue = SDB.Objects("BAFQueue")
    Dim i : i = Int(queue.Item("i"))
    Dim data : Set data = SDB.Objects("BAFData"&i)
    Dim fso : Set fso = CreateObject("Scripting.FileSystemObject")
    'create binary file
    On Error Resume Next
    Dim bin : Set bin = CreateObject("ADODB.Stream")
    If Not (bin Is Nothing) Then
      If Debug Then Call out("Saving "&data.Item("dest"))
      bin.Type = 1
      bin.Open
      'copy from buffer in chunks
      Dim done : done = False
      Do
        SDB.ProcessMessages
        Dim buf : buf = inet.GetChunk(512,1)
        If UBound(buf) < 1 Then
          done = True
        Else
          bin.Write buf
        End If
      Loop Until done
      'save binary file
      bin.SaveToFile data.Item("dest"),2
      bin.Close
      Set bin = Nothing
    End If
    'error handling
    If Err.Number > 0 And Debug Then
      Call out("***"&Err.Number&" - "&Err.Description&" ("&Err.Source&")")
      If Err.Number = 3004 Then
        If fso.FileExists(data.Item("dest")) Then
          Call out("***Y")
        Else
          Call out("***N")
        End If
      End If
    End If    
    On Error GoTo 0
    'check file downloaded
    Dim save : save = True
    If fso.FileExists(data.Item("dest")) Then
      Dim jpg : Set jpg = fso.GetFile(data.Item("dest"))
      If Not (jpg Is Nothing) Then
        'check file size
        If jpg.Size = 0 Then
          Set jpg = Nothing
        Else
          'check html file
          Dim txt : Set txt = fso.OpenTextFile(data.Item("dest"),1,False)
          Dim str : str = UCase(txt.Read(6))
          If (str = "<HTML>") Or (str = "<!DOCT") Then
            Set jpg = Nothing
          End If        
          txt.Close
          Set txt = Nothing
        End If
      End If      
      'remove empty file
      If jpg Is Nothing Then
        fso.DeleteFile(data.Item("dest"))
        save = False
        If Debug Then Call out("File '"&data.Item("dest")&"' was empty or html.")
      End If
    Else
      save = False
      If Debug Then Call out("File '"&data.Item("dest")&"' was missing.")
    End If
    'update tracks
    If save Then
      If Debug Then Call out("Saving complete")
      Call postDownload(i)
    Else
      'update document
      Dim doc : Set doc = SDB.Objects("BatchArtFinderDoc")
      If Not (doc Is Nothing) Then
        Dim div : Set div = doc.getElementById("dim"&i)
        If Not (div Is Nothing) Then
          div.innerHTML = "Error :("
        End If
      End If
    End If
    'update queue
    queue.Item("i") = i+1
    Set SDB.Objects("BAFinet") = Nothing
  End If
End Sub

Sub postDownload(i)
  Dim fso : Set fso = CreateObject("Scripting.FileSystemObject")
  Dim data : Set data = SDB.Objects("BAFData"&i)
  Dim dest : dest = data.Item("dest")
  'calculate storage
  Storage = SDB.IniFile.IntValue("BatchArtFinder","Storage")-1
  If Storage < 5 Then
    If Debug Then Call out("Tagging "&dest)
    Dim sto : sto = Storage
    If Storage = 2 Or Storage = 3 Then
      sto = 0
    End If
    'add to tags
    Dim list,j
    Dim alb : Set alb = SDB.Tools.AlbumByName(data.Item("oartist"),data.Item("oalbum"))
    If alb Is Nothing Then
      Set alb = SDB.Database.QuerySongs("AND (Songs.ID="&data.Item("id")&")")
      If Not (alb.EOF) Then     
        Set list = SDB.NewSongList
        list.Add(alb.Item)
        Set alb = Nothing
        Call puttrackart(list.Item(0),dest,sto)
        list.UpdateAll
        list.Item(0).WriteTags
      End If
    Else
      Set list = alb.Tracks
      For j = 0 To list.Count-1
        Call puttrackart(list.Item(j),dest,sto)
      Next
      list.UpdateAll
      For j = 0 To list.Count-1
        list.Item(j).WriteTags
      Next      
    End If
    'add again if both
    If Storage = 2 Then
      If alb Is Nothing Then
        Set alb = SDB.Database.QuerySongs("AND (Songs.ID="&data.Item("id")&")")
        If Not (alb.EOF) Then     
          Set list = SDB.NewSongList
          list.Add(alb.Item)
          Set alb = Nothing
          Call puttrackart(list.Item(0),dest,1)
          list.UpdateAll
          list.Item(0).WriteTags
        End If
      Else
        Set list = alb.Tracks
        For j = 0 To list.Count-1
          Call puttrackart(list.Item(j),dest,1)
        Next
        list.UpdateAll
        For j = 0 To list.Count-1
          list.Item(j).WriteTags
        Next        
      End If    
    End If
    'delete if necessary
    If Storage = 3 Then
      If Not (dest = getnonejpg()) Then
        fso.DeleteFile(dest)
      End If
    End If
    If Debug Then Call out("Tagging complete")
  End If
  'update document
  Dim doc : Set doc = SDB.Objects("BatchArtFinderDoc")
  If Not (doc Is Nothing) Then
    Dim div : Set div = doc.getElementById("sav"&i)
    If Not (div Is Nothing) Then
      div.Checked = False
      Set div = doc.getElementById("img"&i)
      If Not (div Is Nothing) Then
        Dim div2 : Set div2 = doc.getElementById("dim"&i)
        If Not (div2 Is Nothing) Then
          If Left(div.innerHTML,1) = "<" Then
            div2.innerHTML = "Saved!"
          Else
            div.innerHTML = "<img src='"&dest&"'>"
            div2.innerHTML = "Saved!"
          End If
        End If
      End If
    End If
  End If  
End Sub

Sub queueIsEmpty()
  'reset todo list
  Dim todo : Set todo = SDB.Objects("BatchArtFinderToDo")
  Dim queue : Set queue = SDB.Objects("BAFQueue")
  todo.Item("max") = queue.Item("old")
  Set SDB.Objects("BAFQueue") = Nothing
  
  'perform save action
  Select Case SDB.IniFile.IntValue("BatchArtFinder","SaveMode")
    Case 1 'next
      Call NextClick(Nothing)
    Case 2 'exit
      Dim form : Set form = SDB.Objects("BatchArtFinderForm")
      form.Common.Visible = False
      Call FormClose(form)      
    Case 3 'trynext
      Call TryClick()      
    Case Else 'nothing
      Dim but : Set but = SDB.Objects("BAFClickedBtn")
      but.Common.Enabled = True
      Set SDB.Objects("BAFClickedBtn") = Nothing
  End Select
End Sub

Sub puttrackart(itm,dest,sto)
  Dim pics : Set pics = itm.AlbumArt
  If pics Is Nothing Then Exit Sub
  
  'add image to artwork
  Dim img : Set img = pics.AddNew
  img.PicturePath = dest
  img.Description = "Added by Trixmoto's BatchArtFinder script"
  img.ItemStorage = sto
  img.ItemType = 3
  pics.UpdateDB
End Sub

Function getnonejpg()
  getnonejpg = Replace(Script.ScriptPath,"BatchArtFinder.vbs","none.jpg")
End Function

Function gettime(sec)
  Dim min : min = 0
  sec = Int(sec)
  Do While sec > 59 
    sec = sec - 60
    min = min + 1
  Loop
  If sec < 10 Then
    gettime = min&":0"&sec
  Else
    gettime = min&":"&sec
  End If
End Function

Function mapname(name)
  Dim i : i = 0 
  Dim nameUTF8 : nameUTF8 = UTF8_Encode(name)
  For i = 1 To Len(nameUTF8)
    Dim c : c = Asc(Mid(nameUTF8,i))
    If (c>32 And c<48) Or (c>57 And c<65) Or (c>90 And c<97) Or (c>122 And c<127) Then
      mapname = mapname&"%"&CStr(Hex(c))
    Else
      mapname = mapname&Chr(c)
    End If
  Next
  mapname = Replace(mapname," ","+")
End Function

Sub ClearData
  Dim paths : Set paths = SDB.Objects("BatchArtFinderPaths")
  Dim todo : Set todo = SDB.Objects("BatchArtFinderToDo")
  Dim max : max = Int(todo.Item("max"))
  If max > 0 Then
    Dim i : i = 0
    For i = 1 To max
      Set SDB.Objects("BAFData"&i) = Nothing
    Next
    Set SDB.Objects("BatchArtFinderToDo") = Nothing
    Set SDB.Objects("BatchArtFinderForm") = Nothing
    Set SDB.Objects("BatchArtFinderDoc") = Nothing
    Set SDB.Objects("BatchArtFinderShell") = Nothing    
    Set SDB.Objects("BatchArtFinderPanel") = Nothing 
    Set SDB.Objects("BAFQueue") = Nothing
    Set SDB.Objects("BAFCurrentSong") = Nothing
    Set SDB.Objects("BatchArtFinderPaths") = Nothing
    Set SDB.Objects("BatchArtFinderDrp1") = Nothing
    Set SDB.Objects("BatchArtFinderBtn1") = Nothing
    Set SDB.Objects("BatchArtFinderBtn2") = Nothing
    Set SDB.Objects("BatchArtFinderBtn3") = Nothing
  End If  

  Dim fso : Set fso = SDB.Tools.FileSystem
  If fso.FileExists(paths.Item("htm")) Then
    Call fso.DeleteFile(paths.Item("htm"))
  End If    
End Sub

Sub PauseClick(ClickedBtn)
  'toggle button
  Dim stat1,stat2,i
  If ClickedBtn.Caption = "Pause" Then
    ClickedBtn.Caption = "Resume"
    stat1 = "[Queued]"
    stat2 = "[Paused]"
  Else
    ClickedBtn.Caption = "Pause"
    stat1 = "[Paused]"
    stat2 = "[Queued]"    
  End If
  
  'update images
  Dim doc : Set doc = SDB.Objects("BatchArtFinderDoc")
  Dim todo : Set todo = SDB.Objects("BatchArtFinderToDo")
  Dim max : max = Int(todo.Item("max"))
  For i = 1 To max
    Dim div : Set div = doc.getElementById("img"&i)
    If Not (div Is Nothing) Then
      If div.innerHTML = stat1 Then
        div.innerHTML = stat2
      End If     
    End If    
  Next
End Sub

Sub NextClick(ClickedBtn)
  'calculate next start
  Dim todo : Set todo = SDB.Objects("BatchArtFinderToDo")
  Dim str : str = todo.Item(Int(todo.Item("max")))
  Dim id : id = 0
  If IsNumeric(str) Then
    id = Int(str)
  Else
    id = 0 - Int(Mid(str,2))
  End If
  SDB.IniFile.StringValue("BatchArtFinder","StartID") = "*"&id
  
  'close current form
  Dim form : Set form = SDB.Objects("BatchArtFinderForm")
  form.Common.Visible = False
  Call FormClose(form)  
  
  'start next batch
  Call DoBatchArtFinder(Nothing)
End Sub

Sub SourceClick(Drp)
  Dim todo : Set todo = SDB.Objects("BatchArtFinderToDo")
  If todo.Item("drp") = "" Then
    todo.Item("drp") = Drp.ItemIndex
    Dim Tmr : Set Tmr = SDB.CreateTimer(500)
    Script.RegisterEvent Tmr, "OnTimer", "DelaySelect"
  End If
End Sub

Sub DelaySelect(Tmr)
  Script.UnregisterEvents Tmr
  
  'change source
  Dim todo : Set todo = SDB.Objects("BatchArtFinderToDo")
  SDB.IniFile.IntValue("BatchArtFinder","Source") = Int(todo.Item("drp"))
  todo.Item("drp") = ""
  Dim itm : Set itm = SDB.Objects("BAFCurrentSong")
  
  'close current form
  Dim form : Set form = SDB.Objects("BatchArtFinderForm")
  form.Common.Visible = False
  Call FormClose(form)  
  
  'start next batch
  Call DoBatchArtFinder(itm)
End Sub

Sub InitSheet(Sheet)
  Dim ini : Set ini = SDB.IniFile
  Storage = ini.IntValue("BatchArtFinder","Storage")
  ImageSize = ini.IntValue("BatchArtFinder","ImageSize")
  FileMask = ini.StringValue("BatchArtFinder","FileMask")
  Extra = ini.StringValue("BatchArtFinder","Extra")
  Maximum = ini.IntValue("BatchArtFinder","Maximum")
  Singles = ini.IntValue("BatchArtFinder","Singles")
  Include = ini.IntValue("BatchArtFinder","Include")
  PanelType = ini.IntValue("BatchArtFinder","PanelType")
  Toolbar = ini.IntValue("BatchArtFinder","Toolbar")
  Default = ini.IntValue("BatchArtFinder","Default")
  CheckAll = ini.IntValue("BatchArtFinder","CheckAll")
  Source = ini.IntValue("BatchArtFinder","Source")
  MinW = ini.IntValue("BatchArtFinder","MinW")
  MaxW = ini.IntValue("BatchArtFinder","MaxW")
  MinH = ini.IntValue("BatchArtFinder","MinH")
  MaxH = ini.IntValue("BatchArtFinder","MaxH")
  ConfMode = ini.IntValue("BatchArtFinder","ConfMode")
  SaveMode = ini.IntValue("BatchArtFinder","SaveMode")
  AutoMode = ini.IntValue("BatchArtFinder","AutoMode")
  SetSize = ini.IntValue("BatchArtFinder","SetSize")
  SetW = ini.IntValue("BatchArtFinder","SetW")
  SetH = ini.IntValue("BatchArtFinder","SetH")
  FileTypes = ini.StringValue("BatchArtFinder","FileTypes")
  Persist = ini.IntValue("BatchArtFinder","Persist")
  
  Dim ui : Set ui = SDB.UI
  Dim edt : Set edt = ui.NewLabel(Sheet)
  edt.Common.SetRect 5, 10, 50, 20
  edt.Caption = "Storage type:"
  edt.Autosize = False
  
  Set edt = ui.NewDropDown(Sheet)
  edt.Common.SetRect 85, 7, 150, 20
  edt.Common.ControlName = "BAFStorage" 
  edt.Style = 2
  edt.AddItem("None")
  edt.AddItem("Save in tag")
  edt.AddItem("Save as link")
  edt.AddItem("In tag and as link")
  edt.AddItem("In tag only (delete image)")
  edt.AddItem("Download file (don't tag)")
  edt.ItemIndex = Storage
  
  Set edt = ui.NewLabel(Sheet)
  edt.Common.SetRect 260, 10, 50, 20
  edt.Caption = "Panel type:"
  edt.Autosize = False
  
  Set edt = ui.NewDropDown(Sheet)
  edt.Common.SetRect 330, 7, 100, 20
  edt.Common.ControlName = "BAFPanelType"
  edt.Style = 2
  edt.AddItem("Stay-on-top")
  edt.AddItem("Normal")
  edt.AddItem("Dockable")
  edt.ItemIndex = PanelType - 1
  
  Set edt = ui.NewLabel(Sheet)
  edt.Common.SetRect 5, 35, 50, 20
  edt.Caption = "Source:"
  edt.Autosize = False
  
  Set edt = ui.NewLabel(Sheet)
  edt.Common.SetRect 260, 35, 50, 20
  edt.Caption = "Image size:"
  edt.Autosize = False
  
  Set edt = ui.NewDropDown(Sheet)
  edt.Common.SetRect 330, 32, 100, 20
  edt.Common.ControlName = "BAFImageSize"
  edt.Style = 2
  edt.AddItem("Small")
  edt.AddItem("Medium")
  edt.AddItem("Large")
  edt.AddItem("All")
  edt.ItemIndex = ImageSize  
  
  Set edt = ui.NewButton(Sheet)
  edt.Common.SetRect 60, 32, 20, 20
  edt.Common.ControlName = "BAFMore"
  edt.Caption = "\/"
  edt.UseScript = Script.ScriptPath
  edt.OnClickFunc = "MoreOptions"  
  
  Set edt = ui.NewDropDown(Sheet)
  edt.Common.SetRect 85, 32, 150, 20
  edt.Common.ControlName = "BAFSource"
  edt.Style = 2
  edt.AddItem("images.google.com")
  edt.AddItem("www.rateyourmusic.com")
  edt.AddItem("www.juno.co.uk")
  edt.AddItem("albumart.org")
  edt.AddItem("ws.audioscrobbler.com")
  edt.AddItem("localhost")
  '!
  edt.ItemIndex = Source 
  edt.UseScript = Script.ScriptPath
  edt.OnSelectFunc = "ShowImageSize" 
  Call ShowImageSize(edt)

  Set edt = ui.NewLabel(Sheet)
  edt.Common.SetRect 5, 60, 50, 20
  edt.Caption = "File mask:"
  edt.Autosize = False
  
  Set edt = ui.NewEdit(Sheet)
  edt.Common.SetRect 85, 57, 345, 20
  edt.Common.ControlName = "BAFFileMask"
  edt.Common.Hint = "Valid fields: <path> <artist> <album>"
  edt.Text = FileMask

  Set edt = ui.NewLabel(Sheet)
  edt.Common.SetRect 5, 85, 50, 20
  edt.Caption = "Extra:"
  edt.Autosize = False
  
  Set edt = ui.NewEdit(Sheet)
  edt.Common.SetRect 85, 82, 345, 20
  edt.Common.ControlName = "BAFExtra"
  edt.Common.Hint = "Part of SQL Where statement: 'AND ...'"
  edt.Text = Extra
  
  Set edt = ui.NewButton(Sheet)
  edt.Common.SetRect 430, 82, 20, 20
  edt.Common.ControlName = "BAFClear"&i
  edt.Common.Hint = "Test query"
  edt.Caption = "?"
  edt.UseScript = Script.ScriptPath
  edt.OnClickFunc = "TestExtra"  
  
  Set edt = ui.NewLabel(Sheet)
  edt.Common.SetRect 5, 110, 50, 20
  edt.Caption = "Batch limit:"
  edt.Autosize = False
  
  Set edt = ui.NewSpinEdit(Sheet)
  edt.Common.SetRect 85, 107, 100, 20
  edt.Common.ControlName = "BAFMaximum"
  edt.MinValue = 0
  edt.MaxValue = 9999
  edt.Value = Maximum
  
  Set edt = ui.NewLabel(Sheet)
  edt.Common.SetRect 190, 110, 100, 20
  edt.Caption = "(None=0)"
  edt.Autosize = False  
 
  Set edt = ui.NewLabel(Sheet)
  edt.Common.SetRect 260, 110, 100, 20
  edt.Caption = "File types:"
  edt.Autosize = False  
  
  Set edt = ui.NewEdit(Sheet)
  edt.Common.SetRect 330, 107, 100, 20
  edt.Common.ControlName = "BAFFileTypes"
  edt.Common.Hint = "Comma separated list including: jpg,png,gif,bmp"
  edt.Text = FileTypes
  
  Set edt = ui.NewLabel(Sheet)
  edt.Common.SetRect 5, 135, 50, 20
  edt.Caption = "Min. width:"
  edt.Autosize = False
  
  Set edt = ui.NewSpinEdit(Sheet)
  edt.Common.SetRect 85, 132, 100, 20
  edt.Common.ControlName = "BAFMinW"
  edt.MinValue = 0
  edt.MaxValue = 9999
  edt.Value = MinW
  
  Set edt = ui.NewLabel(Sheet)
  edt.Common.SetRect 260, 135, 50, 20
  edt.Caption = "Max. width:"
  edt.Autosize = False  
  
  Set edt = ui.NewSpinEdit(Sheet)
  edt.Common.SetRect 330, 132, 100, 20
  edt.Common.ControlName = "BAFMaxW"
  edt.MinValue = 0
  edt.MaxValue = 9999
  edt.Value = MaxW  
  
  Set edt = ui.NewLabel(Sheet)
  edt.Common.SetRect 5, 160, 50, 20
  edt.Caption = "Min. height:"
  edt.Autosize = False
  
  Set edt = ui.NewSpinEdit(Sheet)
  edt.Common.SetRect 85, 157, 100, 20
  edt.Common.ControlName = "BAFMinH"
  edt.MinValue = 0
  edt.MaxValue = 9999
  edt.Value = MinH     
  
  Set edt = ui.NewLabel(Sheet)
  edt.Common.SetRect 260, 160, 50, 20
  edt.Caption = "Max. height:"
  edt.Autosize = False
  
  Set edt = ui.NewSpinEdit(Sheet)
  edt.Common.SetRect 330, 157, 100, 20
  edt.Common.ControlName = "BAFMaxH"
  edt.MinValue = 0
  edt.MaxValue = 9999
  edt.Value = MaxH  
  
  Set edt = ui.NewCheckbox(Sheet)
  edt.Common.SetRect 5, 185, 250, 20
  edt.Common.ControlName = "BAFSingles"
  edt.Caption = "Include non-album tracks as albums"
  If Singles = 1 Then
    edt.Checked = True
  Else
    edt.Checked = False
  End If
  
  Set edt = ui.NewCheckbox(Sheet)
  edt.Common.SetRect 5, 208, 250, 20
  edt.Common.ControlName = "BAFInclude"
  edt.Caption = "Include non-album track titles in url"
  If Include = 1 Then
    edt.Checked = True
  Else
    edt.Checked = False
  End If  
  
  Set edt = ui.NewCheckbox(Sheet)
  edt.Common.SetRect 5, 231, 300, 20
  edt.Common.ControlName = "BAFToolbar"
  edt.Caption = "Add run button to the standard toolbar"
  If Toolbar = 1 Then
    edt.Checked = True
  Else
    edt.Checked = False
  End If
  
  Set edt = ui.NewCheckbox(Sheet)
  edt.Common.SetRect 5, 254, 300, 20
  edt.Common.ControlName = "BAFDefault"
  edt.Caption = "Add default image (none.jpg) to albums without results"
  If Default = 1 Then
    edt.Checked = True
  Else
    edt.Checked = False
  End If  
  
  Set edt = ui.NewCheckbox(Sheet)
  edt.Common.SetRect 5, 277, 300, 20
  edt.Common.ControlName = "BAFCheckAll"
  edt.Caption = "Start all images as ticked to be saved"
  If CheckAll = 1 Then
    edt.Checked = True
  Else
    edt.Checked = False
  End If   
  
  Set edt = ui.NewCheckbox(Sheet)
  edt.Common.SetRect 5, 300, 400, 20
  edt.Common.ControlName = "BAFAutoMode"
  edt.Caption = "Automatically launch script when playing track without artwork"
  If AutoMode = 1 Then
    edt.Checked = True
  Else
    edt.Checked = False
  End If     
  
  Set edt = ui.NewCheckbox(Sheet)
  edt.Common.SetRect 5, 323, 250, 20
  edt.Common.ControlName = "BAFSetSize"
  edt.Caption = "Set image display size ---------------------------->"
  If SetSize = 1 Then
    edt.Checked = True
  Else
    edt.Checked = False
  End If     
  
  Set edt = ui.NewCheckbox(Sheet)
  edt.Common.SetRect 5, 346, 250, 20
  edt.Common.ControlName = "BAFPersist"
  edt.Caption = "Remember prompt values"
  If Persist = 1 Then
    edt.Checked = True
    edt.Common.Hint = GetHint()
  Else
    edt.Checked = False
  End If
  
  Set edt = ui.NewLabel(Sheet)
  edt.Common.SetRect 260, 323, 50, 20
  edt.Caption = "Disp. width:"
  edt.Autosize = False  
  
  Set edt = ui.NewSpinEdit(Sheet)
  edt.Common.SetRect 330, 320, 100, 20
  edt.Common.ControlName = "BAFSetW"
  edt.MinValue = 10
  edt.MaxValue = 999
  edt.Value = SetW 
  
  Set edt = ui.NewLabel(Sheet)
  edt.Common.SetRect 260, 346, 50, 20
  edt.Caption = "Disp. height:"
  edt.Autosize = False
  
  Set edt = ui.NewSpinEdit(Sheet)
  edt.Common.SetRect 330, 343, 100, 20
  edt.Common.ControlName = "BAFSetH"
  edt.MinValue = 10
  edt.MaxValue = 999
  edt.Value = SetH  
  
  Set edt = ui.NewLabel(Sheet)
  edt.Common.SetRect 260, 185, 50, 20
  edt.Caption = "Selection:"
  edt.Autosize = False
  
  Set edt = ui.NewDropDown(Sheet)
  edt.Common.SetRect 330, 182, 100, 20
  edt.Common.ControlName = "BAFConfMode"
  edt.Style = 2
  edt.AddItem("Prompt")
  edt.AddItem("Selected only")
  edt.AddItem("All albums")
  edt.ItemIndex = ConfMode  
  
  Set edt = ui.NewLabel(Sheet)
  edt.Common.SetRect 260, 210, 50, 20
  edt.Caption = "After saving:"
  edt.Autosize = False
  
  Set edt = ui.NewDropDown(Sheet)
  edt.Common.SetRect 330, 207, 100, 20
  edt.Common.ControlName = "BAFSaveMode"
  edt.Style = 2
  edt.AddItem("Do nothing")
  edt.AddItem("Click next")
  edt.AddItem("Close form")
  edt.AddItem("Try next")
  edt.ItemIndex = SaveMode   
  
  Dim pan : Set pan = ui.NewPanel(Sheet)
  pan.Common.Align = 2
  pan.Common.Height = 268
  pan.Common.ControlName = ""
  pan.Common.Visible = False
  Set SDB.Objects("BatchArtFinderPanel") = pan
  
  Set edt = ui.NewLabel(pan)
  edt.Common.SetRect 5, 5, 50, 20
  edt.Caption = "Search string:"
  edt.Autosize = False
  
  Set edt = ui.NewEdit(pan)
  edt.Common.SetRect 85, 2, 345, 20
  edt.Common.ControlName = "BAFLocalMask"
  edt.Common.Hint = "Example: <album>.*"
  edt.Text = ini.StringValue("BatchArtFinder","LocalMask")
  
  Set edt = ui.NewLabel(pan)
  edt.Common.SetRect 5, 30, 50, 20
  edt.Caption = "Search folders:"
  edt.Autosize = False
  
  Set edt = ui.NewLabel(pan)
  edt.Common.SetRect 305, 30, 50, 20
  edt.Caption = "Include subfolders?"
  edt.Autosize = False  
  
  Dim locs : locs = ini.StringValue("BatchArtFinder","Locations")
  Dim i : i = 0
  While (Len(locs) > 0) And (i < 9)
    Dim path : path = locs
    Dim pos : pos = InStr(locs,"|")
    If pos > 0 Then
      path = Left(locs,pos-1)
      locs = Mid(locs,pos+1)
    Else
      locs = ""
    End If
    Dim subs : subs = False
    If Right(path,1) = "*" Then
      subs = True
      path = Left(path,Len(path)-1)
    End If
    i = i + 1
    Set edt = ui.NewButton(pan)
    edt.Common.SetRect 5, (i*25)+27, 20, 20
    edt.Common.ControlName = "BAFBrowse"&i
    edt.Caption = "..."
    edt.UseScript = Script.ScriptPath
    edt.OnClickFunc = "MoreBrowse"    
    Set edt = ui.NewEdit(pan)
    edt.Common.SetRect 25, (i*25)+27, 300, 20
    edt.Common.ControlName = "BAFLocation"&i
    edt.Text = path
    Set edt = ui.NewCheckBox(pan)
    edt.Common.SetRect 330, (i*25)+27, 20, 20
    edt.Common.ControlName = "BAFRecurse"&i
    edt.Caption = ""
    edt.Checked = subs
    If i < 8 Then
      Set edt = ui.NewButton(pan)
      edt.Common.SetRect 380, (i*25)+27, 50, 20
      edt.Common.ControlName = "BAFClear"&i
      edt.Caption = "Clear"
      edt.UseScript = Script.ScriptPath
      edt.OnClickFunc = "MoreClear"
    End If
  WEnd
  
  If i < 8 Then
    i = i + 1
    Set edt = ui.NewButton(pan)
    edt.Common.SetRect 5, (i*25)+27, 20, 20
    edt.Common.ControlName = "BAFBrowse"&i
    edt.Caption = "..."
    edt.UseScript = Script.ScriptPath
    edt.OnClickFunc = "MoreBrowse"  
    Set edt = ui.NewEdit(pan)
    edt.Common.SetRect 25, (i*25)+27, 300, 20
    edt.Common.ControlName = "BAFLocation"&i
    edt.Text = ""
    Set edt = ui.NewCheckBox(pan)
    edt.Common.SetRect 330, (i*25)+27, 20, 20
    edt.Common.ControlName = "BAFRecurse"&i
    edt.Caption = ""
    edt.Checked = False
    If i < 8 Then
      Set edt = ui.NewButton(pan)
      edt.Common.SetRect 380, (i*25)+27, 50, 20
      edt.Common.ControlName = "BAFAdd"&i
      edt.Caption = "More"
      edt.UseScript = Script.ScriptPath
      edt.OnClickFunc = "MoreAdd"
    End If
  End If
End Sub

Sub MoreOptions(but)
  Dim pan : Set pan = SDB.Objects("BatchArtFinderPanel")
  If pan Is Nothing Then
    but.Caption = "X"
    but.Common.Enabled = False
  Else
    If but.Caption = "\/" Then
      but.Caption = "/\" 
      pan.Common.Visible = True
    Else
      but.Caption = "\/"
      pan.Common.Visible = False
    End If
  End If
End Sub

Sub MoreBrowse(but)
  Dim s : s = Mid(but.Common.ControlName,10)
  If IsNumeric(s) Then
    Dim i : i = Int(s)
    Dim edt : Set edt = but.Common.TopParent.Common.ChildControl("BAFLocation"&i)
    If Not (edt Is Nothing) Then
      s = SDB.SelectFolder(edt.Text,"Select folder:")
      If Not (s = "") Then
        edt.Text = s&"\"
      End If
    End If
  End If
End Sub

Sub MoreClear(but)
  Dim s : s = Mid(but.Common.ControlName,9)
  If IsNumeric(s) Then
    Dim i : i = Int(s)
    Dim edt : Set edt = but.Common.TopParent.Common.ChildControl("BAFLocation"&i)
    If Not (edt Is Nothing) Then
      edt.Text = ""
    End If
    Set edt = but.Common.TopParent.Common.ChildControl("BAFRecurse"&i)
    If Not (edt Is Nothing) Then
      edt.Checked = False
    End If
  End If
End Sub

Sub MoreAdd(but)
  Dim s : s = Mid(but.Common.ControlName,7)
  If IsNumeric(s) Then  
    Dim i : i = Int(s)
    but.Common.ControlName = "BAFClear"&i
    but.Caption = "Clear"
    but.OnClickFunc = "MoreClear"    
    i = i + 1
    Dim pan : Set pan = SDB.Objects("BatchArtFinderPanel")
    If Not (pan Is Nothing) Then
      Dim ui : Set ui = SDB.UI
      Dim edt : Set edt = ui.NewButton(pan)
      edt.Common.SetRect 5, (i*25)+27, 20, 20
      edt.Common.ControlName = "BAFBrowse"&i
      edt.Caption = "..."
      edt.UseScript = Script.ScriptPath
      edt.OnClickFunc = "MoreBrowse"  
      Set edt = ui.NewEdit(pan)
      edt.Common.SetRect 25, (i*25)+27, 300, 20
      edt.Common.ControlName = "BAFLocation"&i
      edt.Text = ""
      Set edt = ui.NewCheckBox(pan)
      edt.Common.SetRect 330, (i*25)+27, 20, 20
      edt.Common.ControlName = "BAFRecurse"&i
      edt.Caption = ""
      edt.Checked = False
      If i < 8 Then
        Set edt = ui.NewButton(pan)
        edt.Common.SetRect 380, (i*25)+27, 50, 20
        edt.Common.ControlName = "BAFAdd"&i
        edt.Caption = "More"
        edt.UseScript = Script.ScriptPath
        edt.OnClickFunc = "MoreAdd"
      End If
    End If
  End If
End Sub

Sub TestExtra(but)
  Dim str : str = "test passed."
  Dim typ : typ = mtInformation
  Dim edt : Set edt = but.Common.TopParent.Common.ChildControl("BAFExtra")
  If edt Is Nothing Then
    str = "test error."
    typ = mtError
  Else
    On Error Resume Next
    Dim temp : Set temp = SDB.Database.OpenSQL("SELECT Count(*) AS AlbumCount FROM Albums WHERE Albums.ID>0 "&edt.Text)
    If Err.Number <> 0 Then
      str = "test failed."
      typ = mtError
      Err.Clear
    End If
    On Error Goto 0
  End If
  Call SDB.MessageBox("Batch Art Finder - "&str,typ,Array(mbOk))
End Sub

Sub ShowImageSize(src)
  Dim size : Set size = src.Common.TopParent.Common.ChildControl("BAFImageSize")
  Dim more : Set more = src.Common.TopParent.Common.ChildControl("BAFMore")
  Select Case src.ItemIndex
    Case 0
      size.Common.Enabled = True
      more.Common.Visible = False
    Case 1
      size.Common.Enabled = False
      more.Common.Visible = False
    Case 2
      size.Common.Enabled = True      
      more.Common.Visible = False
    Case 3
      size.Common.Enabled = False      
      more.Common.Visible = False
    Case 4
      size.Common.Enabled = True
      more.Common.Visible = False
    Case 5
      size.Common.Enabled = False      
      more.Common.Visible = True
    '!
  End Select
End Sub

Sub SaveSheet(Sheet)
  Dim ini : Set ini = SDB.IniFile
  ini.IntValue("BatchArtFinder","Storage") = Sheet.Common.ChildControl("BAFStorage").ItemIndex
  ini.IntValue("BatchArtFinder","ImageSize") = Sheet.Common.ChildControl("BAFImageSize").ItemIndex
  ini.StringValue("BatchArtFinder","FileMask") = Sheet.Common.ChildControl("BAFFileMask").Text
  ini.StringValue("BatchArtFinder","Extra") = Sheet.Common.ChildControl("BAFExtra").Text
  ini.IntValue("BatchArtFinder","Maximum") = Sheet.Common.ChildControl("BAFMaximum").Value
  ini.IntValue("BatchArtFinder","PanelType") = Sheet.Common.ChildControl("BAFPanelType").ItemIndex + 1
  ini.StringValue("BatchArtFinder","FileTypes") = Sheet.Common.ChildControl("BAFFileTypes").Text
  
  If Sheet.Common.ChildControl("BAFSingles").Checked Then
    ini.IntValue("BatchArtFinder","Singles") = 1
  Else
    ini.IntValue("BatchArtFinder","Singles") = 0
  End If
  
  If Sheet.Common.ChildControl("BAFInclude").Checked Then
    ini.IntValue("BatchArtFinder","Include") = 1
  Else
    ini.IntValue("BatchArtFinder","Include") = 0
  End If  
  
  Dim but : Set but = SDB.Objects("BAFToolbarButton")
  If Sheet.Common.ChildControl("BAFToolbar").Checked Then
    ini.IntValue("BatchArtFinder","Toolbar") = 1
    but.Visible = True
  Else
    ini.IntValue("BatchArtFinder","Toolbar") = 0
    but.Visible = False
  End If
  
  If Sheet.Common.ChildControl("BAFDefault").Checked Then
    ini.IntValue("BatchArtFinder","Default") = 1
  Else
    ini.IntValue("BatchArtFinder","Default") = 0
  End If  
  
  If Sheet.Common.ChildControl("BAFCheckAll").Checked Then
    ini.IntValue("BatchArtFinder","CheckAll") = 1
  Else
    ini.IntValue("BatchArtFinder","CheckAll") = 0
  End If  
  
  If Sheet.Common.ChildControl("BAFAutoMode").Checked Then
    ini.IntValue("BatchArtFinder","AutoMode") = 1
  Else
    ini.IntValue("BatchArtFinder","AutoMode") = 0
  End If    
  
  ini.IntValue("BatchArtFinder","Source") = Sheet.Common.ChildControl("BAFSource").ItemIndex
  ini.IntValue("BatchArtFinder","MaxW") = Sheet.Common.ChildControl("BAFMaxW").Value
  ini.IntValue("BatchArtFinder","MinW") = Sheet.Common.ChildControl("BAFMinW").Value
  ini.IntValue("BatchArtFinder","MaxH") = Sheet.Common.ChildControl("BAFMaxH").Value
  ini.IntValue("BatchArtFinder","MinH") = Sheet.Common.ChildControl("BAFMinH").Value   
  ini.IntValue("BatchArtFinder","ConfMode") = Sheet.Common.ChildControl("BAFConfMode").ItemIndex
  ini.IntValue("BatchArtFinder","SaveMode") = Sheet.Common.ChildControl("BAFSaveMode").ItemIndex

  If Sheet.Common.ChildControl("BAFSetSize").Checked Then
    ini.IntValue("BatchArtFinder","SetSize") = 1
  Else
    ini.IntValue("BatchArtFinder","SetSize") = 0
  End If      
  ini.IntValue("BatchArtFinder","SetW") = Sheet.Common.ChildControl("BAFSetW").Value
  ini.IntValue("BatchArtFinder","SetH") = Sheet.Common.ChildControl("BAFSetH").Value     
  If Sheet.Common.ChildControl("BAFPersist").Checked Then
    ini.IntValue("BatchArtFinder","Persist") = 1
    Dim hnt : hnt = GetHint()
    If Not (hnt = "") Then
      hnt = Chr(13)&"-"&Chr(13)&hnt
    End If
    SDB.Objects("BAFToolbarButton").Caption = "Batch Art Finder"&hnt    
  Else
    ini.IntValue("BatchArtFinder","Persist") = 0
    ini.StringValue("BatchArtFinder","Replies1") = ""
    ini.StringValue("BatchArtFinder","Replies2") = ""
    ini.StringValue("BatchArtFinder","Replies3") = ""
    SDB.Objects("BAFToolbarButton").Caption = "Batch Art Finder"
    Set SDB.Objects("BatchArtFinderReplies") = CreateObject("Scripting.Dictionary")
  End If        
  
  ini.StringValue("BatchArtFinder","LocalMask") = Sheet.Common.ChildControl("BAFLocalMask").Text   
  Dim locs : locs = ""
  Dim i : i = 1
  Dim edt : Set edt = Sheet.Common.ChildControl("BAFLocation"&i)
  While Not (edt Is Nothing)
    If Not (edt.Text = "") Then
      If locs = "" Then
        locs = edt.Text
      Else
        locs = locs&"|"&edt.Text
      End If
      Set edt = Sheet.Common.ChildControl("BAFRecurse"&i)
      If Not (edt Is Nothing) Then
        If edt.Checked Then
          locs = locs&"*"
        End If
      End If
    End If
    i = i + 1
    Set edt = Sheet.Common.ChildControl("BAFLocation"&i)
  WEnd
  ini.StringValue("BatchArtFinder","Locations") = locs
End Sub

Sub LocalHost(key)
  'get relevant data
  Dim data : Set data = SDB.Objects("BAFData"&key)
  Dim locs : locs = data.Item("url")
  Dim img : img = Int(data.Item("image"))
  If locs = "" Then
    Call ShowLocalResult(key,"",0)
    Exit Sub
  End If
  If data.Item("previous") = "1" Then
    img = img-2
    data.Item("previous") = "0"
  End If
  
  'get location
  Dim recu : recu = False
  Dim path : path = ""
  Dim i : i = InStr(locs,"|")
  If i = 0 Then
    path = locs
    locs = ""
  Else
    path = Left(locs,i-1)
    locs = Mid(locs,i+1)
  End If
  If Right(path,1) = "*" Then
    recu = True
    path = Left(path,Len(path)-1)
  End If
  If Not (Right(path,1) = "\") Then
    path = path&"\"
  End If
  
  'check location
  Dim fso : Set fso = CreateObject("Scripting.FileSystemObject")  
  If fso.FolderExists(path) Then
    'add subfolders
    Dim fold : Set fold = fso.GetFolder(path)
    If recu Then
      Dim subf
      Dim subs : Set subs = fold.SubFolders
      For Each subf In subs
        locs = subf.Path&"*|"&locs
      Next   
    End If  
  
    'calculate mask
    Dim mask : mask = SDB.IniFile.StringValue("BatchArtFinder","LocalMask")
    mask = Replace(mask,"<artist>",data.Item("artist"))
    mask = Replace(mask,"<album>",data.Item("album"))
    mask = fixmask(mask)
    mask = Replace(mask,"*",".*")

    'check files
    i = 0
    Dim filf
    Dim fils : Set fils = fold.Files
    For Each filf In fils
      i = i + 1
      If i > img Then
        If RegExpTest(mask,filf.Name) Then
          'show result
          data.Item("url") = path&"|"&locs
          Call ShowLocalResult(key,filf.Path,i)
          Exit Sub
        End If
      End If 
    Next  
  End If

  'try next location
  data.Item("url") = locs
  data.Item("image") = 0
  Call LocalHost(key)
End Sub

Function ShowLocalResult(key,src,i)
  'get relevant data
  Dim data : Set data = SDB.Objects("BAFData"&key)
  Dim mess : mess = "[No results]"
  ShowLocalResult = False
   
  'update data
  If i > 0 Then
    data.Item("image") = i
    data.Item("source") = src
    mess = "<a href="""&Left(src,InStrRev(src,"/"))&""" target=""_blank""><img src="""&src&""" title="""&src&"""></a>"
    ShowLocalResult = True  
  End If
 
  'update document
  Dim doc : Set doc = SDB.Objects("BatchArtFinderDoc")
  If Not (doc Is Nothing) Then
    Dim div : Set div = doc.getElementById("img"&key)
    If Not (div Is Nothing) Then
      div.innerHTML = mess
    End If
  End If 
  
  'start next search
  Call TriggerNext()
End Function

Function RegExpTest(pat,str)
  Dim objRegEx : Set objRegEx = New RegExp
  objRegEx.IgnoreCase = True
  objRegEx.Pattern = pat
  RegExpTest = objRegEx.Test(str)  
End Function

Function fixmask(str)
  Dim i : i = 0
  Dim special : special = "^$.|{}[]()+?\"
  For i = 1 To Len(str)
    Dim c : c = Mid(str,i,1)
    If InStr(special,c) = 0 Then
      fixmask = fixmask&c
    Else
      fixmask = fixmask&"\"&c
    End If
  Next
End Function

Sub preLocalHost()
  'check document
  Dim doc : Set doc = SDB.Objects("BatchArtFinderDoc")
  If Not (doc Is Nothing) Then
  'get queue data
    Dim fso : Set fso = CreateObject("Scripting.FileSystemObject")
    Dim queue : Set queue = SDB.Objects("BAFQueue")
    Dim min : min = Int(queue.Item("i"))
    Dim max : max = Int(queue.Item("max"))
    Dim i : i = 0
    Dim none : none = ""
    If SDB.IniFile.IntValue("BatchArtFinder","Default") = 1 Then
      none = getnonejpg()
    End If  
    'loop through queue
    For i = min To max
      Dim div : Set div = doc.getElementById("sav"&i)
      If Not (div Is Nothing) Then
        If div.Checked = True Then
          Set div = doc.getElementById("img"&i)
          If Not (div Is Nothing) Then
            Dim data : Set data = SDB.Objects("BAFData"&i)
            If Left(div.innerHTML,1) = "<" Then
              'build path from mask and data
              FileMask = SDB.IniFile.StringValue("BatchArtFinder","FileMask")
              Dim dest : dest = FileMask&data.Item("type")
              dest = Replace(dest,"<path>",data.Item("path"))
              dest = Replace(dest,"<artist>",data.Item("artist"))
              dest = Replace(dest,"<album>",data.Item("album"))
              Dim path : path = Mid(dest,3)
              path = SDB.Tools.FileSystem.CorrectFilename(path)
              data.Item("dest") = Left(dest,2)&path
              Call fso.CopyFile(data.Item("source"),data.Item("dest"),True)
              Call postDownload(i)
            Else
              If Not (none = "") Then
                data.Item("dest") = none
                Call fso.CopyFile(data.Item("source"),none,True)
                Call postDownload(i)
              End If
            End If
          End If
        End If
      End If
      Call SDB.ProcessMessages
    Next
  End If
  'update queue
  queue.Item("i") = max+1
  Call queueIsEmpty
End Sub

Function UTF8_Encode(ByVal UnicodeString)
  Dim i,UnicodeCharCode,UTF8_temp  
  On Error Resume Next
  For i = 1 To Len(UnicodeString)
    UnicodeCharCode = AscW(Mid(UnicodeString,i))
    If UnicodeCharCode < &H80 Then
      UTF8_temp = Chr(UnicodeCharCode)
    ElseIf UnicodeCharCode < &H800 Then
      UTF8_temp = Chr(&H80 + (UnicodeCharCode And &H3F))
      UnicodeCharCode = UnicodeCharCode \ &H40
      UTF8_temp = Chr(&HC0 + (UnicodeCharCode And &H1F)) & UTF8_temp
    Else
      UTF8_temp = Chr(&H80 + (UnicodeCharCode And &H3F))
      UnicodeCharCode = UnicodeCharCode \ &H40
      UTF8_temp = Chr(&H80 + (UnicodeCharCode And &H3F)) & UTF8_temp
      UnicodeCharCode = UnicodeCharCode \ &H40
      UTF8_temp = Chr(&HE0 + (UnicodeCharCode And &HF)) & UTF8_temp
    End If
    UTF8_Encode = UTF8_Encode & UTF8_temp
    If Not (Err.Number = 0) Then
      Err.Clear
      UTF8_Encode = UnicodeString
      Exit For
    End If
  Next
  On Error Goto 0
End Function

Function fixyear(str)
  fixyear = str
  Dim i : i = InStr(fixyear,".")
  If i > 1 Then
    fixyear = Left(fixyear,i-1)
  End If
  While Len(fixyear) < 4
    fixyear = "0"&fixyear
  WEnd
  If fixyear = "0000" Then
    fixyear = "????"
  End If
End Function

Function InFileTypes(arr,src,ext)
  InFileTypes = False
  ext = ""
  Dim i : i = InStrRev(src,".")
  If i > 1 Then
    Dim s : s = UCase(Mid(src,i))
    For i = 0 To UBound(arr)
      If s = "."&arr(i) Then
        InFileTypes = True
        ext = LCase(s)
        Exit Function
      End If
    Next
  End If
End Function

Function GetHint()
  Dim tmp : tmp = ""
  Dim rep : Set rep = SDB.Objects("BatchArtFinderReplies")
  If Not (rep Is Nothing) Then
    If rep.Exists("restrict") Then
      tmp = "Restrict to selected albums: "
      Select Case Int(rep.Item("restrict"))
        Case mrYes
          tmp = tmp&"Yes"
        Case mrAll
          tmp = tmp&"All"
        Case mrCancel
          tmp = tmp&"Cancel"
        Case Else
          tmp = tmp&"("&rep.Item("restrict")&")"
      End Select
    End If
    If rep.Exists("beginning") Then
      If tmp = "" Then
        tmp = "Start from the beginning: "
      Else
        tmp = tmp&Chr(13)&"Start from the beginning: "
      End If
      Select Case Int(rep.Item("beginning"))
        Case mrYes
          tmp = tmp&"Yes"
        Case mrNo
          tmp = tmp&"No"
        Case mrCancel
          tmp = tmp&"Cancel"
        Case Else
          tmp = tmp&"("&rep.Item("beginning")&")"
      End Select
    End If
    If rep.Exists("include") Then
      If tmp = "" Then
        tmp = "Process albums with artwork: "
      Else
        tmp = tmp&Chr(13)&"Process albums with artwork: "
      End If
      Select Case Int(rep.Item("include"))
        Case mrYes
          tmp = tmp&"Yes"
        Case mrNo
          tmp = tmp&"No"
        Case Else
          tmp = tmp&"("&rep.Item("include")&")"
      End Select
    End If      
  End If
  GetHint = tmp
End Function

Sub out(txt)
  Dim pat : Set pat = SDB.Objects("BatchArtFinderPaths")
  If Not (pat Is Nothing) Then
    Dim loc : loc = pat.Item("log")
    If Not (loc = "") Then
      Dim fso : Set fso = CreateObject("Scripting.FileSystemObject")
      Dim logf : Set logf = fso.OpenTextFile(loc,8,True)
      logf.WriteLine(Time&" "&SDB.ToAscii(txt))
      logf.Close
    End If
  End If
End Sub

Sub Install()
  Dim form : Set form = SDB.Objects("BatchArtFinderForm")
  If Not (form Is Nothing) Then
    form.Common.Visible = False  
    form.Common.ControlName = ""
    Set SDB.Objects("BatchArtFinderForm") = Nothing
  End If

  Dim inip : inip = SDB.ApplicationPath&"Scripts\Scripts.ini"
  Dim inif : Set inif = SDB.Tools.IniFileByPath(inip)
  If Not (inif Is Nothing) Then
    inif.StringValue("BatchArtFinder","Filename") = "Auto\BatchArtFinder.vbs"
    inif.StringValue("BatchArtFinder","Procname") = "BatchArtFinder"
    inif.StringValue("BatchArtFinder","Order") = "28"
    inif.StringValue("BatchArtFinder","DisplayName") = "Batch Art Finder"
    inif.StringValue("BatchArtFinder","Description") = "Find album art in batch"
    inif.StringValue("BatchArtFinder","Language") = "VBScript"
    inif.StringValue("BatchArtFinder","ScriptType") = "0"
    SDB.RefreshScriptItems
  End If
  Call OnStartup()
End Sub  